import os

import numpy as np
from six import print_ as print

from .latticeseq_b2 import latticeseq_b2


def qmc(
    func, xmin, xmax, args=(), kwargs={},
    nshifts=8, pow2min=3, pow2max=20,
    chebyshevk=2, abserr=0, relerr=1e-06, maxeval=None,
    showinfo=True):
    """
    Lattice sequence approximation of multivariate integral
    (using latticeseq_b2).

    Inputs:
        func        function to integrate over the n-dimensional rectangular area,
                    if your function can compute multiple point at one time,
                    it should accept x of shape (npts, ndim) and
                    return a numpy 2d array of shape (npts, fdim)
                    otherwise it should accpet x as numpy 1d array of shape (ndim,) and
                    return a scalar or a numpy 1d array of shape (fdim,)

        xmin        whatever can convert to numpy 2d arrays of shape (1, n)
        xmax        whatever can convert to numpy 2d arrays of shape (1, n)
                    *note*
                        maximum number of dimensions for current generating vector
                        is 250
        args        arguments for the integrand function
        kwargs      keyword arguments for the integrand function
        nshifts     number of randomizations for the lattice seq to obtain stderr
        chebyshevk a multiple of the stderr is used to obtain an error estimate
                    the most conservative way is to use the Cheybshev inequality:
                       k  | confidence (= 1 - 1/k^2)
                     -----+------------
                       2  |     75%
                       3  |    ~88%
                       4  |    ~93%
                       5  |    ~96%
                       8  |    ~98%
                    since this holds for any distribution (of the results) with mean
                    and variance these estimates are normally very conservative and
                    so the actual confidence level might be a lot higher than indicated above
        maxeval     if set then we set pow2max such that 2**pow2max * nshifts is as close as possible
    

        example
            qmc(func, array([-3, -2, 1]), array([5, 2, 4]))
            then func should accept numpy array of dimension npts-by-3 or
            numpy 1d array of shape (3,)

            func is called like:
                func(x, a1, a2)    if args=(a1,a2)
                func(x, a1)        if args=(a1,)
                func(x, a1, k1=k1) if args=(a1,) and kwargs={'k1': k1}

        minimum evaluations is 2**pow2min * nshifts
        maximum evaluations is 2**pow2max * nshifts

    Outputs:
        Q           approximation to the integral
        estAbsErr   estimated absolute error: chebyshev_k * stderr(nshifts independent approximations)
        infos       some info as dictionary
            stdQ        standard deviation of Q
            nbFunEvals  number of function evaluate
    """

    if maxeval != None:
        pow2max = min(int(np.rint(np.log2(maxeval/nshifts))), 20)

    xmin = np.array(xmin, ndmin=2)
    xmax = np.array(xmax, ndmin=2)
    scale = xmax - xmin
    jacobian = np.prod(scale)

    ndim = xmin.shape[1] # get ndim
    latgen = latticeseq_b2(s=ndim)

    np.random.seed(1) # Mersenne Twister reset
    shifts = np.random.rand(nshifts, ndim)

    # test vectorized and get fdim
    try:
        fdim = func(np.vstack((xmin,xmax)), *args, **kwargs).shape[1]
        vectorized = True
    except Exception as e: #has a risk if func is for no vectorized but
                         #can accept 2d numpy array
        print('qmc warning: integration function is not vectorized.', e)
        val = func(xmin.ravel(), *args, **kwargs)
        if hasattr(val,'__len__'):
            fdim = len(val)
        else:
            fdim = 1
        vectorized = False
    acc = np.zeros((nshifts,fdim))
    m = 0

    for m in range(0, pow2max + 1):
        # generate an array of 2**(m-1) x s unless m=0, 1 x s
        x = latgen.calc_block(m)
        if vectorized == True:
            npts = x.shape[0]
            x_vec = np.tile(x, (nshifts, 1))
            sh_vec = np.repeat(shifts, npts, axis=0)
            x_sh_vec = (scale * ((x_vec + sh_vec) % 1) + xmin)
            v_vec = func(x_sh_vec, *args, **kwargs)
            acc += jacobian* v_vec.reshape((nshifts, npts, fdim)).sum(axis=1)
        else:
            for xk in x:
                for j in range(nshifts):
                    xshifted = (scale * ((xk + shifts[j,:]) % 1) + xmin).ravel()
                    #force convert to 1d array
                    fval = np.array(func(xshifted, *args, **kwargs), ndmin=1)
                    acc[i] += jacobian * fval
        Q = np.mean(acc/2**m, axis=0)
        stdQ = np.std(acc/2**m, axis=0) / np.sqrt(nshifts)
        if m < pow2min: continue
        if (stdQ * chebyshevk <= abserr).all(): break
        if (stdQ * chebyshevk <= relerr * np.abs(Q)).all(): break
    if showinfo:
        print("QMC Q=", Q, " std=", stdQ, " N=", 2**m * nshifts, \
              " s=", ndim, " relErrReq=", relerr, " absErrReq=", abserr)
    
    #Q = Q
    estAbsErr = chebyshevk*stdQ
    #stdQ = stdQ
    nbFunEvals = 2**m*nshifts
    infos = {'stdQ': stdQ, 'nbFunEvals': nbFunEvals}
    
    return Q, estAbsErr, infos


if __name__ == '__main__':
    import time

    radius = 1.
    xmin = np.array([0, 0, 0], np.float64)
    xmax = np.array([radius, 2*np.pi, np.pi], np.float64)
    def integrand_sphere(x_array):
        r, theta, phi = x_array
        return r**2*np.sin(phi)

    def integrand_sphere_vectorized(x_array):
        r = x_array[:,0:1]
        theta = x_array[:,1:2]
        phi = x_array[:,2:3]
        # r, theta, phi = x_array
        return r**2*np.sin(phi)
 
    # # TOO SLOW without vectorized...
    # v,e = qmc(integrand_sphere, xmin, xmax)
    # print(v,e)
    method = 'qmc'
    abserr = 0
    relerr = 10**-7

    print('Using                 :', method)
    sss = time.time()
    v, e, infos = \
        qmc(
            integrand_sphere_vectorized, xmin, xmax,
            abserr=abserr, relerr=relerr, pow2max=24, showinfo=True
            )
    finfo = {
        'thinksAbsErrReqReached': e < abserr,
        'thinksRelErrReqReached': e < np.abs(v) * relerr,
        'stdQ': infos['stdQ'],
        'nbFunEvals': infos['nbFunEvals'] }
    print('Time                  : %.3g sec' % (time.time() - sss))
    orgprintopts = np.get_printoptions()
    np.set_printoptions(formatter={'float': '{: 10.03e}'.format})
    print('Value                 :', v)
    print('EstAbsErr             :', e)
    for key, val in finfo.items():
        print("%-22s: %s" % (key, val))
    np.set_printoptions(orgprintopts)
    
    # print('sphere volume:', v)
    # print('sphere volume error:', e)
    print('exact sphere volume:', 4/3. * np.pi * radius**3)



# -*- coding: utf-8 -*-
import numpy as np
from scipy.special import gamma, kv

def get_model(m_name):
    'use model name to get model'
    try:
        return eval(m_name)
    except NameError as e:
        #must be do something...
        raise e

def exponentialC(dist, sill, ar, jac=False, jacpar=None):
    '''
    Exponential covariance function

    Input:
    dist      m by n    an 2D array of spatial or S/T distances
    sill      scalar    covariance sill
    ar        scalar    covariance correlation length
    jac       bool      if the return should be jacobian of parameters.
                        the default is False to return the covariance estimation
    jacpar    string    To determine which parameter is used for Jacobian
                        estimation, the default is None.
                        Options are 'sill' and 'ar'
    
    Output:
    cov/jac   m by n    an 2D array of covariance or Jacobian estimation 
                        cooresponding to dist
    '''

    if not jac:
        cov = sill * np.exp(-3 * dist / ar)
        return cov
    else:
        if jacpar == 'sill':
            jac = np.exp(-3 * dist / ar)
        elif jacpar == 'ar':
            jac = sill*3*dist/(ar**2)*np.exp(-3*dist/ar)
        elif jacpar == 'ar2':
            jac = sill*((3*dist/(ar**2))**2-6*dist/(ar**3))*np.exp(-3*dist/ar)      
        return jac

def gaussianC(dist, sill, ar, jac=False, jacpar=None):
    if not jac:
        return sill * np.exp(-3 * (dist / ar)**2)
    else:
        if jacpar == 'sill':
            jac = np.exp(-3 * (dist / ar)**2)
        elif jacpar == 'ar':
            jac = sill * np.exp(-3 * (dist / ar)**2)*(6*(dist**2)/ar**3)
        elif jacpar == 'ar2':
            jac = sill * np.exp(-3*(dist/ar)**2) *\
                ((6*(dist**2)/ar**3)**2 - 18*(dist**2)/(ar**4))
        return jac

def sphericalC(dist, sill, ar, jac=False, jacpar=None):
    value = dist / ar
    if not jac:    
        result = sill * (1- ((3/2.) * value - (1/2.) * (value)**3))
        result[dist > ar] = 0.
        return result
    else:
        if jacpar == 'sill':
            jac = 1 - ((3/2.) * value - (1/2.) * (value)**3)
            jac[dist > ar] = 0.
        elif jacpar == 'ar':
            jac = sill*(3/2.*dist/ar**2-3/2.*dist**3/ar**4)   
            jac[dist > ar] = 0.
        elif jacpar == 'ar':
            jac = sill*(-3.*dist/ar**3-6.*dist**3/ar**5)   
            jac[dist > ar] = 0.      
        return jac  
    
def holecosC(dist, a_half, p_half):
    return a_half * np.cos(np.pi * dist / p_half)

def nuggetC(dist, sill, ar=None, jac=False, jacpar=None):
    if not jac:
        result = np.zeros(dist.shape)
        result[dist == 0] = sill
        return result
    else:
        jac=np.zeros([0,0])
        return jac

def maternC(dist, sill, parms):
    '''
    Matern covariance with practical distance adjustment
    
    C = maternC(dist, sill, parms)
    
    Input
    dist    m by n     an numpy array of S/T distances
    sill    scalar     sill
    parms   list       a list of two components 
                       (correlation length, shape parameter).
                       
    Output
    Cov     m by n     an numpy array of S/T covariances                   
    '''

    ar = parms[0]
    shp = parms[1]
    scale = np.sqrt(2*shp*6)*(3./2)**(1./4./shp)
    sdist = scale*dist
    result =\
        (sill/(2.**(shp-1))/gamma(shp)) * ((sdist/ar)**shp) * kv(shp,sdist/ar)
    result[np.where(sdist==0)] = sill
    return result



# -*- coding: utf-8 -*-
import numpy
from scipy.spatial.distance import cdist


def findpairs(c1, c2):
  '''
  Find pairs of coordinates that are identical for two diferent
  matrices of coordinates.

  SYNTAX :

  [index]=findpairs(c1,c2);

  INPUT :

  c1        n1 by d  numpy matrix of coordinates, where d is the dimension
                     of the space.
  c2        n2 by d  numpy matrix of coordinates.

  OUTPUT :

  index     n by 2   numpy matrix of indices for identical coordinates
                     in c1 and c2. The first column of index 
                     refers to the c1 matrix, whereas the second
                     column of index refers to the c2 matrix.

  '''
  
  try:
    idx=findpairs_1(c1,c2)
  except:
    idx=findpairs_2(c1,c2)
  return idx  

def findpairs_1( c1, c2):
  idx=numpy.where(cdist(c1,c2)==0.0)
  pair_idx_list=numpy.asarray(zip(idx[0],idx[1])) 
  return pair_idx_list

def findpairs_2( c1, c2 ):
    '''return index pair'''

    if c1.shape[0] <= c2.shape[0] :
        smaller_c = c1
        bigger_c = c2
        smaller_is_c1 = True
    else:
        smaller_c = c2
        bigger_c = c1
        smaller_is_c1 = False

    pair_idx_list = []
    for small_idx, row_i in enumerate( smaller_c ):
        big_idx = numpy.where( numpy.all( row_i == bigger_c, axis = 1 ) )[0]
        if len(big_idx):
            pair_idx_list.append( [ small_idx, big_idx[0] ] )

    if pair_idx_list:
        pair_idx_list = numpy.array( pair_idx_list )
        if smaller_is_c1:
            return pair_idx_list
        else:
            pair_idx_list[ :, [ 0, 1 ] ] =  pair_idx_list[ :, [ 1, 0 ] ]
            return pair_idx_list
    else:
        return pair_idx_list

if __name__ == "__main__":
    import time
    c1 = numpy.array([[1,1,1],[1,2,1],[1,1,2.],[2,2,1]])
    c2 = numpy.array([[1,2,5],[2,2,2.],[0,0,1.],[1,0,0],[2,1,2],[2,2,2]])
    res=findpairs( c1, c2 )
    print res
    c1=numpy.random.rand(1000,3)
    c2=numpy.random.rand(1000,3)
    aaa = time.time()
    result1 = findpairs(c1,c2)
    print 'neighbor_kd Time: ', time.time() - aaa
    aaa = time.time()
    result2 = findpairs_1(c1,c2)
    print 'neighbor_kd Time: ', time.time() - aaa


# -*- coding: utf-8 -*-
"""
Created on Thu Nov 13 23:03:23 2014

@author: hdragon689
"""

import datetime
import pandas
try:
    import rpy2.robjects as robjects
    from rpy2.robjects.packages import importr
except ImportError, e:
    print ('Cannot import modeul "rpy2.robjects", try to install "rpy2" first.')
    raise e
import matplotlib.pyplot as plt
from numpy import asarray, ceil, isnan


# import r functions
zoo_ = importr('zoo')
ts_ = robjects.r['ts']
stl_ = robjects.r['stl']
naaction_ = robjects.r['na.approx'] # Generic functions for replacing each 
                                      # NA with interpolated values.


def stl(data, ns, np=None, nt=None, nl=None, isdeg=0, itdeg=1, ildeg=1,
        nsjump=None, ntjump=None, nljump=None, robust=False, ni=2, no=0, fulloutput=False):
  """
  Seasonal-Trend decomposition procedure based on LOESS
  
  data : pandas.Series or 1D numpy array, in which np is required 
         for numpy array input
  
  ns : int
  	Length of the seasonal smoother.
  	The value of  ns should be an odd integer greater than or equal to 3.
  	A value ns>6 is recommended. As ns  increases  the  values  of  the
  	seasonal component at a given point in the seasonal cycle (e.g., January
  	values of a monthly series with  a  yearly cycle) become smoother.
  
  np : int
  	Period of the seasonal component.
  	For example, if  the  time series is monthly with a yearly cycle, then
  	np=12.
  	If no value is given, then the period will be determined from the
  	``data`` timeseries.
  
  nt : int
  	Length of the trend smoother.
  	The  value  of  nt should be an odd integer greater than or equal to 3.
  	A value of nt between 1.5*np and 2*np is  recommended. As nt increases,
  	the values of the trend component become  smoother.
  	If nt is None, it is estimated as the smallest odd integer greater
  	or equal to ``(1.5*np)/[1-(1.5/ns)]``
  
  nl : int
  	Length of the low-pass filter.
  	The value of nl should  be an odd integer greater than or equal to 3.
  	The smallest odd integer greater than or equal to np is used by default.
  
  isdeg : int
  	Degree of locally-fitted polynomial in seasonal smoothing.
  	The value is 0 or 1.
  
  itdeg : int
  	Degree of locally-fitted polynomial in trend smoothing.
  	The value is 0 or 1.
  
  ildeg : int
  	Degree of locally-fitted polynomial in low-pass smoothing.
  	The value is 0 or 1.
  
  nsjump : int
  	Skipping value for seasonal smoothing.
  	The seasonal smoother skips ahead nsjump points and then linearly
  	interpolates in between.  The value  of nsjump should be a positive
  	integer; if nsjump=1, a seasonal smooth is calculated at all n points.
  	To make the procedure run faster, a reasonable choice for nsjump is
  	10%-20% of ns. By default, nsjump= 0.1*ns.

  ntjump : int
  	Skipping value for trend smoothing. If None, ntjump= 0.1*nt
  
  nljump : int
  	Skipping value for low-pass smoothing. If None, nljump= 0.1*nl
  
  robust : bool
  	logical indicating if robust fitting be used in the loess procedure      
  
  ni :int
  	Number of loops for updating the seasonal and trend  components.
  	The value of ni should be a positive integer.
  	See the next argument for advice on the  choice of ni.
  	If ni is None, ni is set to 2 for robust fitting, to 5 otherwise.

  no : int
  	Number of iterations of robust fitting. The value of no should
  	be a nonnegative integer. If the data are well behaved without
  	outliers, then robustness iterations are not needed. In this case
  	set no=0, and set ni=2 to 5 depending on how much security
  	you want that  the seasonal-trend looping converges.
  	If outliers are present then no=3 is a very secure value unless
  	the outliers are radical, in which case no=5 or even 10 might
  	be better.  If no>0 then set ni to 1 or 2.
  	If None, then no is set to 15 for robust fitting, to 0 otherwise.
  
  fulloutput : bool
  	If True, a dictionary holding the full output of the original R routine
  	will be returned.
  
  returns

  data : pandas.DataFrame or 2D numpy array 
  	A panda.dataframe has the seasonal, trend, and remainder components. 
    The output format is consistent with the one of the input data. 
    If numpt array is used, the three columns are in order 
    for seasonal, trend, and remainder
  
  Note: this code is downloaded from https://gist.github.com/andreas-h/7808564 
  and modified by H-L Yu 2015/06/09
  """         

  # make sure that data doesn't start or end with nan
  #    _data = data.copy()
  #    _data = _data.dropna()
  # TODO: account for non-monthly series
  # Mark by H-L  
  #  _idx = pandas.date_range(start=_data.index[0], end=_data.index[-1],
  #                          offset=pandas.datetools.MonthBegin())
  #    _idx = pandas.date_range(start=_data.index[0], end=_data.index[-1])                        
  #    data = pandas.Series(index=_idx)
  #    data[_data.index] = _data
 
  # zoo package contains na.approx


 
    
  # find out the period of the time series
  if np is None:
    try: 
      freq=data.index.freqstr[0]
      if freq=='M':
        np = 12
      if freq=='D':
        np = 365
      if freq=='W':
        np = 52
    except:
        np=12   # if no np is provided and also can not be recognized by data
                # itself. The data is asssumed to be np==12
        
        # The following comment-out block is from the original code
        # TODO: find out the offset of the Series, and set np accordingly
        #if isinstance(data.index.offset, pandas.core.datetools.MonthEnd):
        #    np = 12
        #else:
        #    raise NotImplementedError()
    # fill default values
      
#  if np is 12:
#    start = robjects.IntVector([data.index[0].year, data.index[0].month])
#    ts = ts_(robjects.FloatVector(asarray(data)), start=start, frequency=np)
  
  # convert data to R object    
  ts = ts_(robjects.FloatVector(asarray(data)), start=1, frequency=np, \
          deltat=1)

  if robust:
    ni=1; no=15;
  else:
    ni=2; no=0;

  if nt is None:
    nt = robjects.rinterface.R_NilValue
  else:
    nt = nt + 1 if nt % 2 == 0 else nt # to assure nt is an odd number

  if nl is None:
    nl = np if np % 2 is 1 else np + 1 
    
  if type(ns)==str and (ns=='per' or ns=='periodic'):
    if isnan(data).any():        
      if ntjump is None:
        ntjump = robjects.rinterface.R_NilValue
      if nljump is None:
        nljump = robjects.rinterface.R_NilValue
      if nsjump is None:
        nsjump = robjects.rinterface.R_NilValue
      result = stl_(ts, ns, isdeg, nt, itdeg, nl, ildeg, nsjump, ntjump, nljump,
                    False, ni, no, naaction_)    
    else:
      result = stl_(ts, ns)
  else:
    if type(nt)==robjects.rinterface.Sexp:
      nt = ceil((1.5 * np) / (1 - (1.5 / ns)))      
    if ntjump is None:
      ntjump = ceil(nt / 10.)
    if nljump is None:
      nljump = ceil(nl / 10.) 
    if nsjump is None:
      nsjump = ceil(ns / 10.)
    
    result = stl_(ts, ns, isdeg, nt, itdeg, nl, ildeg, nsjump, ntjump, nljump,
                  False, ni, no, naaction_)    
 
  res_ts = asarray(result[0])
    
  try:
    res_ts = pandas.DataFrame({"seasonal" : pandas.Series(res_ts[:,0],
                                                    index=data.index),
                               "trend" : pandas.Series(res_ts[:,1],
                                                    index=data.index),
                               "remainder" : pandas.Series(res_ts[:,2],
                                                    index=data.index)})
  except:
    return res_ts #, data
    raise
#        res_ts = pandas.DataFrame({"seasonal" : pandas.Series(index=data.index),
#                                   "trend" : pandas.Series(index=data.index),
#                                   "remainder" : pandas.Series(index=data.index)})
 
  if fulloutput:
    return {"time.series" : res_ts,
            "weights" : result[1],
            "call" : result[2],
            "win" : result[3],
            "deg" : result[4],
            "jump" : result[5],
            "ni" : result[6],
            "no" : result[7]}
  else:
    return res_ts


def stl_plot(data, res, tME=None,title=None):
  '''
  Plot the data and the extracted trend, seasonal, and residual components from
  stl analysis
  
  SYNTAX: stl_plot(data, res, tME=None, title=None)  
  
  INPUT: 
  data    pandas.Series       time series data of the original data 
                              or 1D numpy array
  res     pandas.DataFrame    time series of trend, seasonal, and residual 
                              components. This input is supposed to be the 
                              output of stl function 
  tME     1D numpy array      time index for data and res        
  title   string              title of this figure                            
  '''  

  if tME==None:   
    try:
      tME=data.index
    except:
      tME=np.arange(1,data.size)  
    
  f, axarr = plt.subplots(4, sharex=True)
  if title:
    axarr[0].set_title(title)
  axarr[0].plot(tME, data)
  axarr[0].set_ylabel('Original Data')
  try: # try dataframe
    axarr[1].plot(tME, res['trend'])
    axarr[1].set_ylabel('trend')
    axarr[2].plot(tME, res['seasonal'])
    axarr[2].set_ylabel('seasonal')
    axarr[3].plot(tME, res['remainder'])
    axarr[3].set_ylabel('remainder')
  except: # do numpy array
    axarr[1].plot(tME, res[:,1])
    axarr[1].set_ylabel('trend')
    axarr[2].plot(tME, res[:,0])
    axarr[2].set_ylabel('seasonal')
    axarr[3].plot(tME, res[:,2])
    axarr[3].set_ylabel('remainder')  
  plt.show()  


if __name__ == "__main__":
    
    import numpy as np
    
    data = np.arange(85.) / 12.
    data = np.sin(data * (2*np.pi))
    data += np.arange(85.) / 12. * .5
    data += .1 * np.random.randn(85)
    idx = pandas.date_range(start=datetime.datetime(1999,1,1), \
        end=datetime.datetime(2006,2,1),freq='M')
    data = pandas.Series(data, index=idx)
 
    res = stl(data, np=12, ns=24) #, nt=11)
    
    stl_plot(data,res)



# -*- coding: utf-8 -*-
from six.moves import range
import numpy as np
from scipy.interpolate import griddata

from ..general.coord2dist import coord2dist
from ..stats.eof import eof
from ..stats.stl import stl
from ..stest import idw


#try:
#    from pylab import griddata as pylab_griddata #qgis
#except ImportError:
#    from matplotlib.pylab import griddata as pylab_griddata #OSGEO4W qgis

def _tricube(x):
    w=np.zeros(x.size)
    xx=np.abs(x)
    idx=np.where(np.bitwise_and(xx>=0,xx<1))
    w[idx]=(1-xx[idx]**3)**3
    return w

def _bicube(x):
    w=np.zeros(x.size)
    idx=np.where(np.abs(x)<1)
    w[idx]=(1-np.abs(x[idx])**2)**2
    return w 
def _exp(x):
    w=np.zeros(x.size)
    w=np(-x)
    return w   

def stmean( grid_s, grid_t, grid_z, pars=None,kernel='tricube', DataObj = None ):
    '''
% stmean                  - Estimates space/time mean values from measurements
%
% Assuming a separable additive space/time mean trend model, this 
% function calculates the spatial mean component ms and temporal mean 
% component mt of space/time random field Z, using measurements at 
% fixed measuring  sites cMS and fixed measuring events tME. 
% The spatial mean component ms is obtained by averaging the measurements
% at each measuring sites. Then a smoothed spatial mean component mss
% is obtained by applying a specified spatial filter to ms.
% Similarly mt is obtained by averaging the measurement for each
% measuring event, and a smoothed temporal mean component mts is obtained
% by applying the same function of temporal filter to mt.
% Then the space/time mean trend is simply given by 
% mst(s,t)=mss(s)+mts(t)-mean(mts)
%
% SYNTAX :
%
% [ms,mss,mt,mts,stmean]=stmean(cMS,tME,Z,pars,DataObj=None);
%
% INPUTS :  
%  cMS    nMS by 2   matrix of spatial x-y coordinates for the nMS monitoring
%                    sites
%  idMS   nMS by 1   vector with a unique id number for each monitoring site
%  tME    1 by nME   vector with the time of the measuring events
%  Z      nMS by nME matrix of measurements for Z at the nMS monitoring
%                    sites and nME measuring events. Z may have NaN values.
%  pars   1 by 4     parameters to smooth the spatial and temporal average
%                    p[0]=dNeib  distance (radius) of spatial neighborhood
%                    p[1]=tNeib  time (radius) of temporal neighborhood
                     p[2]=ar  spatial range of kernel smoothing function 
%                    p[3]=at  temporal range of kernel smoothing function
     method string     specified kernel function, including exp, tricube, bicube 
                                         functions. default is tricube function. 

% OUTPUT :
%
%  ms     nMS by 1   vector of spatial average
%  mss    nMS by 1   vector of smoothed spatial average
%  mt     1 by nME   vector of temporal average
%  mts    nMS by 1   vector of smoothed temporal average   
    '''

#    if not DataObj:
#        from nousedataobj import NoUseDataObj
#        DataObj = NoUseDataObj()
    if DataObj:    
        title = DataObj.getProgressText()    
        DataObj.setProgressRange(0,len(grid_s))
        DataObj.setCurrentProgress(0, title + "\n- By STMean...")

    grid_t=np.reshape(grid_t,(1,grid_t.size))  
    grid_t=grid_t.astype(np.float64)

    mask_grid_z = np.ma.masked_array(grid_z,np.isnan(grid_z))
    mean_s = np.array( mask_grid_z.mean( axis = 1 ) , ndmin = 2).T
    mean_t = np.array( mask_grid_z.mean( axis = 0 ) , ndmin = 2)
#  mean_st = mask_grid_z.mean()


    mss=np.zeros(mean_s.shape)
    mts=np.zeros(mean_t.shape)

    s_dist=coord2dist(grid_s,grid_s)
    t_dist=coord2dist(grid_t.T,grid_t.T,1) 
    if pars is None:
        dNeib=np.max(s_dist)*0.4
        tNeib=np.max(t_dist)*0.4
        ar=dNeib
        at=tNeib    
    else:
        dNeib=pars[0]
        tNeib=pars[1]
        ar=pars[2]
        at=pars[3]
    
    if kernel is 'exp':
        fun='_exp'
    elif kernel is 'tricube':
        fun='_tricube'
    elif kernel is 'bicube':
        fun='_bicube'
        
    for i in range(grid_s.shape[0]):
        ids=np.where(s_dist[i]<=dNeib)
        w=eval(fun+'(s_dist[i][ids]/ar)')#_tricube(s_dist[i]/dNeib)
        w=w/np.sum(w)  # normalize
        mss[i]=(w*mean_s.flat[ids].flat[:]).sum()
        
    for j in range(grid_t.size):
        idt=np.where(t_dist[j]<=tNeib)
        w=eval(fun+'(t_dist[j][idt]/at)')#w=_tricube(t_dist[j]/tNeib)
        w=w/np.sum(w)  # normalize
        mts[0,j]=(w*mean_t.flat[idt].flat[:]).sum()

#  grid_trend = mss + mts - mean_st    
#  grid_trend[np.where(np.isnan(grid_z))] = np.nan 
            
    return mean_s,mss,mean_t,mts
        
#    print grid_z
#    print mean_s
#    print mean_t
#    print grid_trend
#    print mean_s + mean_t - mean_st

def stmean_stl( grid_s, grid_t, grid_z, pars=None, kernel='tricube', DataObj = None ):
    '''
% stmean_stl                  - Estimates space/time mean values from measurements
%
% Assuming a separable additive space/time mean trend model, this 
% function calculates the spatial mean component ms and temporal mean 
% component mt of space/time random field Z, using measurements at 
% fixed measuring  sites cMS and fixed measuring events tME. 
% The spatial mean component ms is obtained by averaging the measurements
% at each measuring sites. Then a smoothed spatial mean component mss
% is obtained by applying an exponential spatial filter to ms.
% In this function, a smoothed temporal mean component mts is obtained
% by applying the stl function to extract the periodic trend of mt. 
% Then the space/time mean trend is simply given by 
% mst(s,t)=mss(s)+mts(t)-mean(mts)
%
% SYNTAX :
%
% [ms,mss,mt,mts,stmean]=stmean(cMS,tME,Z,pars,DataObj=None);
%
% INPUTS :  
%  cMS    nMS by 2   matrix of spatial x-y coordinates for the nMS monitoring
%                    sites
%  idMS   nMS by 1   vector with a unique id number for each monitoring site
%  tME    1 by nME   vector with the time of the measuring events
%  Z      nMS by nME matrix of measurements for Z at the nMS monitoring
%                    sites and nME measuring events. Z may have NaN values.
%  pars   1 by 2     parameters to smooth the spatial and temporal average
%                    p[0]=dNeib  distance (radius) of spatial neighborhood
%                    p[1]=np     the time span for seasonal periodic variation 
                                                                 (default=12)
                                         p[2]=ar  spatial range of kernel smoothing function             
%
% OUTPUT :
%
%  ms     nMS by 1   vector of spatial average
%  mss    nMS by 1   vector of smoothed spatial average
%  mt     1 by nME   vector of temporal average
%  mts    nMS by 1   vector of smoothed temporal average   

    Note: See stats.stl 
    '''

#    if not DataObj:
#        from nousedataobj import NoUseDataObj
#        DataObj = NoUseDataObj()
    if DataObj:    
        title = DataObj.getProgressText()    
        DataObj.setProgressRange(0,len(grid_s))
        DataObj.setCurrentProgress(0, title + "\n- By STMean...")

    grid_t=np.reshape(grid_t,(1,grid_t.size))  
    grid_t=grid_t.astype(np.float64)

    mask_grid_z = np.ma.masked_array(grid_z,np.isnan(grid_z))
    mean_s = np.array( mask_grid_z.mean( axis = 1 ) , ndmin = 2).T
    mean_t = np.array( mask_grid_z.mean( axis = 0 ) , ndmin = 2)
#  mean_st = mask_grid_z.mean()

    if kernel is 'exp':
        fun='_exp'
    elif kernel is 'tricube':
        fun='_tricube'
    elif kernel is 'bicube':
        fun='_bicube'
        
    mss=np.zeros(mean_s.shape)
    mts=np.zeros(mean_t.shape)

    s_dist=coord2dist(grid_s,grid_s)
    if pars is None:
        dNeib=np.max(s_dist)*0.4
        np=12
    else:
        dNeib=pars[0]
        np=pars[1]
        ar=pars[2]
        
    for i in range(grid_s.shape[0]):
        ids=np.where(s_dist[i]<=dNeib)
        w=eval(fun+'(s_dist[i][ids]/ar)')#_tricube(s_dist[i]/dNeib)
        w=w/np.sum(w)  # normalize
        mss[i]=(w*mean_s.flat[ids].flat[:]).sum()
        
    mts=stl(mean_t.ravel(),ns='per',np=np)
            
    return mean_s,mss,mean_t,mts  

def stmean_eof(grid_s, grid_t, grid_z, n=5, norm=0, norms_direc=0, method='svd'):

    '''
    Remove mean trend with Empirical Orthogonal Function
    '''

    L, lambdaU2, PC, EOFs, ECs, error, norms = eof(grid_z, n, norm, norms_direc, method)  
    meantrend = np.dot(ECs,EOFs.T)

    return meantrend

def stmeaninterp(grid_s, grid_t, grid_z, est_grid_s, est_grid_t,
                             method='idw', pars=None, smooth=True, DataObj = None):
    '''
    Calculates space/time interpolation of mean trend values using 
    an additive separable s/t mean trend model.
    Using the spatial smoothed mean component ms  and the temporal
    smoothed mean component mt obtained by the function mst from
    from measurements at monitoring sites coordinates cMS and measuring
    events times tME, the s/t mean trend at cooridnate cI and time cI
    is given by
                 mst(s,t)= ms(cI) + mt(tI) - mean(mt),
    where ms(cI) is interpolated from ms at the monitoirng sites, and 
    mt(tI) is interpolated from the mt at the measuring events tME.  

    SYNTAX:
    mstI = stmean_est(cMS, tME, data, xyi, ti, 
                                        method='idw', pars=None, smooth=True, DataObj = None):
    Input:
    
    cMS     m by 2      2D np array of spatial coordinate of the monitors                              
    tME     1 by n      2D np array of times of observed events
    data    m by n      2D np array of observations
    xyi     si by 2     2D np array for spatial coordinates of si interpolated 
                                            locations
    ti      1 by ti     2D np array for times to be interpolated
    method  string      method for spatial trend interpolation. 
                                            Default is idw and griddata is another options including 
                                            linear, cubic, and nearest 
    pars    1 by 2      parameters to smooth the spatial and temporal average
                                            p[0]=dNeib  distance (radius) of spatial neighborhood
                                            p[1]=tNeib  time (radius) of temporal neighborhood 
                                            Default is the 2/5 of the spatial and temporal ranges are 
                                            used as neighborhoods, respectively.
    smooth  bool        optioal. If the smoothed spatial and temporal means are used.
                                            Default is True.                   
    
    Output:    
    mstI    si by ti    2D np array for space-time interpolation
    '''
#    if not DataObj:
#        from nousedataobj import NoUseDataObj
#        DataObj = NoUseDataObj()
    if DataObj:        
        title = DataObj.getProgressText()
        DataObj.setProgressRange(0,len(grid_s))
        DataObj.setCurrentProgress(0, title + "\n- By STMean...")    
        
    # grid_t can be np.datetime or other datetime formats
    # Create the ordinal values for grid_t(tME) for the following operations 
    # by H-L Yu   
        
    # TO-DO add try except for error capture from the wrong input format  
    
    grid_t=np.reshape(grid_t,(1,grid_t.size))  
    grid_t=grid_t.astype(np.float64)
    est_grid_t=np.reshape(est_grid_t,(1,est_grid_t.size))
    est_grid_t=est_grid_t.astype(np.float64) 
    
    ms,mss,mt,mts = stmean(grid_s,grid_t,grid_z,pars)
    if smooth:
        mean_s=mss
        mean_t=mts
    else:
        mean_s=ms
        mean_t=mt
    
    mask_grid_z = np.ma.masked_array(grid_z,np.isnan(grid_z))
#  mean_s = np.array( mask_grid_z.mean( axis = 1 ) , ndmin = 2).T
#  mean_t = np.array( mask_grid_z.mean( axis = 0 ) , ndmin = 2)
    mean_st = mask_grid_z.mean()
        
    if method=='linear':
        mean_s_est = griddata(grid_s, mean_s,est_grid_s)
        mean_s_est[np.isnan(mean_s_est)]=np.mean(mean_s) 
        #Set the nan values to average of mean_s
    elif method=='nearest':
        mean_s_est = griddata(grid_s, mean_s,est_grid_s,method='nearest')
    elif method=='cubic':
        mean_s_est = griddata(grid_s, mean_s,est_grid_s,method='cubic')
    else:
        temp_x,temp_y = map(np.array,zip(*grid_s))
        temp_x_est, temp_y_est = map(np.array,zip(*est_grid_s))
        mean_s_est = idw.idw_est( temp_x, temp_y, mean_s.T[0], temp_x_est, temp_y_est, power = 2 )
    
    mean_s_est=mean_s_est.ravel()  
#    mean_s_est = pylab_griddata(temp_x,temp_y,mean_s.T[0],
#                                temp_x_est,temp_y_est,interp = 'nn')
        #temp_x_est and temp_y_est is not mono increasing
        #i don't know whether there is a bug in it
#    
#    mean_s_est = mean_s_est.diagonal()
    mean_s_est = np.array( mean_s_est,ndmin=2 ).T
    # enforce the temporally linear interpolation
    est_tmin=np.min(np.hstack([est_grid_t[0],grid_t[0]]))
    est_tmax=np.max(np.hstack([est_grid_t[0],grid_t[0]]))
    data_tgrid=np.hstack([est_tmin,grid_t[0],est_tmax])
    data_tmean=np.hstack([mean_t[0][0],mean_t[0],mean_t[0][-1]])
    mean_t_est = np.array(np.interp(est_grid_t[0],data_tgrid,data_tmean),ndmin=2)
        
        #est_z_2d = pylab_griddata(pylab_x, pylab_y, point_value,est_x_2d, est_y_2d,interp = 'nn')
                
    grid_trend_est = mean_s_est + mean_t_est - mean_st
        
    return grid_trend_est
        
        
        

if __name__ == "__main__":
        grid_s=np.array([[1,3.],[1,8],[4,1],[3,2]])
        grid_t=np.array([[1,3,5,7,9.]])
        grid_z = np.array([[1,np.nan,3.,4,5],
                                                    [5,6,1,7,8],
                                                    [1,np.nan,4,2,5],
                                                    [5,2,6,3,1.]])
        
        grid_s_est=grid_s#np.array([[1,6.],[1,6]])[::-1]
        grid_t_est=grid_t#np.array([[1]])

        ms,mss,mt,mts = stmean(grid_s,grid_t,grid_z)
        print ms
        print mss
        print mt
        print mts
        grid_trend = stmeaninterp(grid_s,grid_t,grid_z,grid_s_est,grid_t_est)
        print grid_trend


# -*- coding: utf-8 -*-
import numpy


#only accept order = numpy.nan or 0
def designmatrix( c, order ):
	
	if numpy.isnan(order):
		return numpy.array([],ndmin=2).reshape(c.shape[0],0), numpy.array([],ndmin = 2)
	else:
		n, nd = c.shape
		X = numpy.ones((n, 1))
		index = numpy.zeros((2, 1))
		if nd == 1:
			return X, index[0,:]
		else:
			return X, index
	
	# order = numpy.array([ [order, order] ])
	# if ~numpy.isnan( order[0][0] ) or ~numpy.isnan( order[0][1] ):
	# 	X = numpy.ones((n, 1))
	# 	index = numpy.zeros((2, 1))
	# if ~numpy.isnan( order[0][0] ):




# -*- coding: utf-8 -*-
"""
Data conversion between space/time vector and space/time grid formats

Created on Sat Jun 27 11:40:16 2015

@author: hdragon689
"""

from six.moves import range
import numpy as np
import pandas as pd


def valstv2stg(ch, z, cMS=None, tME=None):
  '''
  Converts the values of a space/time variable from a s/t vector 
  format (i.e. the variable z is listed as a vector of n values)
  to a grid format (i.e. the variable Z is given as a nMS by nME matrix 
  corresponding to nMS Monitoring Sites and nME Measuring Events).
  Use help stgridsyntax for information on the s/t grid format.
 
  SYNTAX :
 
  [Z,cMS,tME,nanratio]=valstv2stg(ch,z,cMS,tME);
 
  INPUT :
 
  ch         n by d+1     matrix of space/time coordinates for spatial domain of dimension d
  z          n by 1       vector of field value at coordinate ch
  cMS        nMS by 2     optional matrix of spatial coordinates for the nMS Measuring Sites
  tME        1 by nME     optional vector of times of the tME Measuring Events
 
  OUTPUT :
 
  Z          nMS by nME   matrix of values for the variable Z corresponding to 
                          nMS Monitoring Sites and nME Measuring Event
  cMS        nMS by d     matrix of spatial coordinates for the nMS Measuring Sites
  tME        1 by nME     vector of times of the tME Measuring Events
  nanratio   scalar       ratio of the NaNs in Z (0<=nanratio<=1) 
 
  NOTE : 
 
  cMS and tME can be provided as input if they are both known.  In that case ch 
  must be a nMS*nME by 3 matrix of the points corresponding to nMS Monitoring Sites
  and nME Measuring Events, listed with space cycling quicker then time.
  '''
  
  if 'pandas' in str(type(ch)):
    ch=ch.values
  if 'pandas' in str(type(z)):
    z=z.values
    
  cols=['x','y','t','z']
  data=np.hstack((ch,z.reshape(z.size,1)))
  datadf=pd.DataFrame(data,columns=cols)  
  datadf['x']=datadf['x'].astype(np.float)
  datadf['y']=datadf['y'].astype(np.float)
  datadf['z']=datadf['z'].astype(np.double)
  dtable=pd.pivot_table(datadf, values=datadf.columns[3], index=['y', 'x'], columns=['t']) 
  # cMS
  cMS_=zip(*np.array(dtable.index))
  cMS_=np.array([np.asarray(cMS_[0]),np.asarray(cMS_[1])])
  cMS_=cMS_.T.astype(np.float)
  cMS_=cMS_[:,[1,0]]
  
  #tME
  try:
    tME_=np.array(dtable.columns).astype(ch[0,2].dtype)
  except AttributeError:
    tME_=np.array(dtable.columns).astype(type(ch[0,2]))

  # Z
  Z_=dtable.values.astype(np.double)  
  
  nt=Z_.shape[1]
  
  if cMS is not None:
    Z=[]
    for i in range(cMS.shape[0]):
      ii=np.where(np.logical_and(cMS_[:,0]==cMS[i,0],cMS_[:,1]==cMS[i,1]))
      Z.append(Z_[ii,:].reshape(nt))
    Z=np.asarray(Z,dtype=np.double) 
  else:
    cMS=cMS_
    Z=Z_
  if tME is not None:
    tidx=[]
    for j,tMEi in enumerate(tME):
      tidx.append(np.where(tME==tMEi))
    tidx=np.asarray(tidx).reshape(tME.size)  
    Z=Z[:,tidx]
  else:
    tME=tME_    
        
  # nonlocations
  nanloc=zip(np.where(np.isnan(Z))[0],np.where(np.isnan(Z))[1])
  
  return Z, cMS, tME, nanloc
    
def valstg2stv(Z, cMS, tME):
  '''
  Converts the coordinates and values of a space/time variable 
  from a grid format (i.e. the variable Z is given as a nMS by nME matrix  
  corresponding to nMS Measuring Sites and nME Measuring Events),
  to a s/t vector format (i.e. the variable z is listed as a vector of nMS*nME values,
  corresponding to points with space/time coordinates, where the spatial coordinate
  cycle quicker than the time coordinates).
 
  SYNTAX :
 
  [ch,z]=valstg2stv(Z,cMS,tME);
 
  INPUT :
 
  Z        nMS by nME       matrix of values for the variable Z corresponding to 
                            nMS Monitoring Sites and nME Measuring Event
  cMS      nMS by 2         matrix of 2D spatial coordinates for the nMS Measuring Sites
  tME      1 by nME         vector of times of the tME Measuring Events
 
  OUTPUT :
 
  ch       nMS*nME by 3     matrix of space time coordinates, listing the space/time
                            locations of the points corresponding to nMS Monitoring Sites
                            and nME Measuring Event (space cycles quicker then time) 
  z        nMS*nME by 1     vector of values for the variable Z corresponding to the
                            s/t points ch
  '''
  
  nc=cMS.shape[0]
  nt=tME.size
  
  zh=(Z.T).reshape(nc*nt,1)
  ch=np.asarray(zip(np.tile(cMS[:,0],nt),np.tile(cMS[:,1],nt), \
                  tME.repeat(nc)))
                  
  return ch, zh
  
if __name__ == "__main__":
  import time
  
  data='../examples/Data/GeoData.xls'      
  datadf=pd.ExcelFile(data).parse('Sheet1',header=None)
  ch=datadf.iloc[:,0:3].values
  z=datadf.iloc[:,4].values.reshape(ch.shape[0],1)
  
  stime=time.time()
  Z1,cMS,tME,nanloc=valstv2stg(ch,z)
  print(time.time()-stime)
  
  ch2,z2=valstg2stv(Z1,cMS,tME)
  
  Z2,cMS2,tME2,nanloc=valstv2stg(ch2,z2,cMS[3:,:],tME[3:])
  Z3,cMS3,tME3,nanloc=valstv2stg(ch2,z2)
  
  id=np.where((Z1!=Z2))
  print(id)



# -*- coding: utf-8 -*-
import numpy

from six.moves import range

from ..general.isspacetime import isspacetime
from ..general import bobyqa as bobyqa

from ..general.coord2K import coord2K, coord2dist
from ..models.covmodel import get_model



def _par2op(covparam):
    pars = []
    for i in range(len(covparam)):
        for j in range(len(covparam[0])):
            if j is not 0:
                pars.append(covparam[i][j][0])
            else:
                pars.append(covparam[i][j])
    indices = [i for i, e in enumerate(pars) if e != None]
    return numpy.array(pars), indices

def _op2covpar(pars_slice, indices,covmodel):
    pars = [None] * (indices[-1]+1)
    for i, e in enumerate(indices):
        pars[e] = pars_slice[i]
    covparam = []
    nm = range(len(covmodel))
    k = 0
    for m in nm:
        covparam.append([pars[k],pars[k+1:k+2],pars[k+2:k+3]])
        k = len(covmodel) * (m+1)
    return covparam

def _covpar2par(covparam):    
    '''This function transforms data formats from regular covariance model to 
    the form used for nlopt. 
       
    Remark: The regular covariance model can refer to covmodeldef function
    '''                 
    fit_params=[]
    nm=len(covparam)
    npars=numpy.zeros(nm,dtype=numpy.int)
    npart=numpy.zeros(nm,dtype=numpy.int)
    for k in range(nm):
        for i,par in enumerate(covparam[k]):
            try:
                for j in range(len(par)):
                    if not (par[j] is None):
                        fit_params.append(par[j])
                if i==1:
                    if par[0] is not None:
                        npars[k]=len(par)
                else:
                    if par[0] is not None:
                        npart[k]=len(par)
            except:
                fit_params.append(par)  
    fit_params=numpy.array(fit_params)      
    return fit_params,npars,npart    


def _par2covpar(param,ns,nt,covmodel):   
    '''This function transforms data formats from the form used for nlopt to 
    the form in the regular covariance model
       The nlopt input format is an 1-D numpy array, in which the covariance 
       parameters are listed in the form of
       [s1, r11, r12, t11, t12, s2, r2, t2, .....] 
       where 
       si is sill of nested model i, 
       rki is the kth spatial range parameters of nested spatial model i
       tki is the kth temporal range parameters of nested temporal model i
    ''' 
    covparam=[]
    k=0
    for m in range(len(covmodel)):
        if any(ns)>0 and any(nt)>0:
            covparam.append([param[k],param[k+1:k+ns[m]+1],
                     param[k+ns[m]+1:k+ns[m]+nt[m]+1]])
            k=k+ns[m]+nt[m]+1 
        elif all(nt)==0:
            covparam.append([param[k],param[k+1:k+ns[m]+1]])
            k=k+ns[m]+1
        elif all(ns)==0:
            covparam.append([param[k],param[k+1:k+nt[m]+1]])
            k=k+nt[m]+1
             
    return covparam

def _par2covSTpar(param,ns,nt):
    covparamS=[]
    covparamT=[]
    nm=len(ns)
    k=0
    for m in range(nm):
        if any(ns)>0 and any(nt)>0:
            covparamS.append([param[k],param[k+1:k+ns[m]+1]])
            covparamT.append([param[k],param[k+ns[m]+1:k+ns[m]+nt[m]+1]])
            k=k+ns[m]+nt[m]+1 
        elif all(nt)==0:
            covparamS.append([param[k],param[k+1:k+ns[m]+1]])
            k=k+ns[m]+1
        elif all(ns)==0:
            covparamT.append([param[k],param[k+1:k+nt[m]+1]])
            k=k+nt[m]+1
             
    return covparamS,covparamT           
  
def _covjac(pars, ns, nt, covmodel, ch):
    '''
    Obtain the Jacobian of the covariance with each of parameters
    Namely, the first derivatives of the covariance with respect to parameters
    '''
    
    covparam = _par2covpar(pars,ns,nt,covmodel)
    isST, isSTsep, model_res = isspacetime(covmodel)
    if isST:
        if isSTsep:
            modelS, modelT = model_res
            dist_s = coord2dist(ch[:, 0:2], ch[:, 0:2])
            dist_t = coord2dist(ch[:, 2:3], ch[:, 2:3])
            jac = []
        
            for model_s, model_t, param_i in zip(modelS, modelT, covparam):
                sill, param_s, param_t = param_i
                model_s = get_model(model_s)
                model_t = get_model(model_t)      
                jac.append(model_s(dist_s, 1., param_s) * model_t(dist_t, 1., param_t))
                jacar=model_s(dist_s, sill, param_s, jac=True, jacpar='ar')
                if jacar.size>0:
                    jac.append(jacar)  
                jacar=model_t(dist_s, sill, param_t, jac=True, jacpar='ar')     
                if jacar.size>0:
                    jac.append(jacar)  
        else:
            (modelS,) = model_res
            #To be implemented
            return
    else:        
        jac=[]
        dist_s = coord2dist(ch, ch)
        (modelS,) = model_res
        for model_s, param_i in zip(modelS, covparam):
            sill, param_s = param_i
            model_s = get_model(model_s)
            jac.append(model_s(dist_s, 1., param_s))  
            jacar=model_s(dist_s, sill, param_s, jac=True, jacpar='ar')
            if jacar.size>0:
                jac.append(jacar)  
        
    return jac     
  
def _covjac2(pars, ns, nt, covmodel, ch):
    '''
    Obtain the second derivatives of the covariance with every of parameters
    '''    
    covparam = _par2covpar(pars, ns, nt, covmodel)
    isST, isSTsep, model_res = isspacetime(covmodel)
    if isST:
        if isSTsep:
            modelS, modelT = model_res
            dist_s = coord2dist(ch[:, 0:2], ch[:, 0:2])
            dist_t = coord2dist(ch[:, 2:3], ch[:, 2:3])
            jac2 = [[None]*pars.size for i in range(pars.size)]
            k = 0            
            for model_s, model_t, param_i in zip(modelS, modelT, covparam):
                sill, param_s, param_t = param_i
                model_s = get_model(model_s)
                model_t = get_model(model_t)  
                parnum = len(param_i)
                jac2[k][k] = numpy.zeros(dist_s.shape) # dsds
                # dsdar
                jac2[k][k+1] = jac2[k+1, k]=\
                  model_s(dist_s, 1., param_s, jac=True, jacpar='ar')*model_t(dist_t, 1., param_t) 
                # dsdat
                jac2[k][k+2] = jac2[k+2,k]=\
                  model_s(dist_s, 1., param_s)*model_t(dist_s, 1., param_t, jac=True, jacpar='ar')
                jac2[k+1][k+1] = model_s(dist_s, sill, param_s, jac=True, jacpar='ar2')*model_t(dist_t, 1., param_t)
                jac2[k+2][k+2] = model_s(dist_s, 1., param_s)*model_t(dist_s, sill, param_t, jac=True, jacpar='ar2')
                k=k+parnum
        else:
            (modelS,) = model_res
            #To be implemented
            return
    else:
        (modelS,) = model_res      
        jac2 = [[None]*pars.size for i in range(pars.size)]
        dist_s = coord2dist(ch, ch)
        k = 0
        for model_s, param_i in zip(modelS, covparam):
            parnum = 1+len(param_i[1])
            sill, param_s = param_i
            model_s = get_model(model_s)
            jac2[k][k] = numpy.zeros(dist_s.shape) # dsds
            for j in range(1, 1+len(param_i[1])):
                # dsdar
                jac2[k][k+j]=jac2[k+j][k]=\
                  model_s(dist_s, 1., param_s, jac=True, jacpar='ar')   
                jac2[k+j][k+j]= model_s(dist_s, sill, param_s, jac=True, jacpar='ar2')
            for m in range(k,k+1+len(param_i[1])):  
                for i in range(k+parnum,pars.size):
                    jac2[m][i]=jac2[i][m]=numpy.zeros(dist_s.shape)
            k=k+parnum
        
    return jac2       
  
def mlecovfitv(ch,zh,covmodel,covparam):
    '''
    Maximum likelihood method for vector format data
    
    opt_param=mlecovfit_v(ch,zh,covmodel,covparam0)
    
    Input
    
    ch          n by d      array consisting of observation locations
    zh          n by 1      array containing obseved values
    covmodel    m           list of m nested covariance models in each of 
                            which the spatial and temporal components are put 
                            in a list as [covmodelS,covmodelT] 
    covparam    m           list of covariance parameters for m covmodels in 
                            each component the parameters are listed as 
                            [sill,[covparamS1,covparamS2,..],
                            [covparamT1,covparamT2,..]] 
    ''' 
    def llikjac(pars,ns,nt):

        covparam = _par2covpar(pars,ns,nt,covmodel)
        jac=_covjac(pars,ns,nt,covmodel,ch)   
        V,_=coord2K(ch,ch,covmodel,covparam)  
        Viy=numpy.linalg.solve(V,zh)
        invV=numpy.linalg.inv(V)
        jacv=numpy.zeros(pars.size)
        for k in range(len(jac)):     
            jacv[k]=0.5*numpy.trace(invV.dot(jac[k])) \
                    -0.5*Viy.T.dot(jac[k]).dot(Viy)#/(zh.T.dot(Viy))

        return jacv  
    
    def hess(pars,ns,nt):
        '''
        Hessain estimation with reference in 
        Kitanidis P., and R. Lane. 1985. Maximum likelihood parameter estimation of 
        hydrological spatial processes by the Gauss-Newton method. Journal of 
        hydrology 79. 
        
        Hessian is calculated by its simplication from (11) to (12) in the paper
        Note that this implementation assumes the mean is constant of zero
        
        '''    
    
        covparam = _par2covpar(pars,ns,nt,covmodel)
        jac=_covjac(pars,ns,nt,covmodel,ch)
        #jac2=_covjac2(pars,ns,nt,covmodel,ch)
        V,_=coord2K(ch,ch,covmodel,covparam) 
        V[numpy.where(V/V[0,0]<10e-6)]=0
        #Viy=numpy.linalg.solve(V,zh)
        invV=numpy.linalg.inv(V)
        hes=numpy.zeros((pars.size,pars.size))
        for j in range(pars.size):
            for k in range(j,pars.size):
                comp1=-0.5*numpy.trace(invV.dot(jac[j]).dot(invV).dot(jac[k]))  
                #comp2=0.5*numpy.trace(invV.dot(jac2[j][k]))
                #comp4=Viy.T.dot(jac[j]).dot(invV).dot(jac[k]).dot(Viy)
                #comp5=-0.5*Viy.T.dot(jac2[j][k]).dot(Viy)
                hes[j,k]=-comp1#comp1+comp2+comp4+comp5
                hes[k,j]=hes[j,k]
        
        return hes    
        
    #  def Fishermat(pars,ns,nt):        
    def loglik(pars,ns,nt):
        if any(pars<=0):
            llik=100000000000
            return llik
        
        covparam = _par2covpar(pars,ns,nt,covmodel)
        V, V_list = coord2K(ch, ch, covmodel, covparam)
        # covariance tapering
        V[numpy.where(V/V[0,0]<10e-6)]=0
        Lv=numpy.linalg.cholesky(V)
        logdetV=2.*numpy.sum(numpy.log(numpy.diag(Lv)))
        #detV = numpy.linalg.det(V)
        n=len(zh)
        Viy=numpy.linalg.solve(V,zh)
        ytViy=zh.T.dot(Viy)
        llik=n/2*numpy.log(2*numpy.pi)+0.5*logdetV+0.5*ytViy    
        return llik    
    
    pars,ns,nt=_covpar2par(covparam)
    zh=zh.reshape(zh.size,)
    #  V=numpy.zeros((ch.size,ch.size))
    #  Viy=numpy.zeros((ch.size,1))
    #  global V, Viy
    #pars, indices = _par2op(covparam)
    #pars_slice = numpy.take(pars,indices)
    if type(ch[0,-1])==numpy.datetime64:
        ch[:,-1]=numpy.double(numpy.asarray(ch[:,-1],dtype='datetime64'))
        ch=ch.astype(numpy.double)
  
    #  _llikjac = lambda pars,ns,nt: llikjac(pars,ns,nt)
    #  _hess = lambda pars,ns,nt: hess(pars,ns,nt)
    #result = op.minimize(loglik, pars_slice, args=(indices),
    #                       options={'maxiter': 10000},method='BFGS')
  
 
    low_bnd=numpy.ones(pars.size)*numpy.finfo(float).eps
    up_bnd=numpy.array([k for k in pars*3])
    args=[ns,nt]
    result, opt_val=bobyqa.bobyqa(loglik, pars, args, low_bnd, up_bnd )

    #  bnds=[(numpy.finfo(float).eps,k) for k in pars*3]   
    #  result=op.differential_evolution(loglik,bnds, args=(ns,nt), maxiter=5)  
    ##  result = op.minimize(loglik, result['x'], args=(ns,nt),jac=_llikjac,hess=_hess,
    ##                       options={'gtol': 1e-6, 'disp': True},method='BFGS')
    #  if not result['success']:                     
    #  result = op.minimize(loglik, pars, args=(ns,nt),jac=_llikjac,hess=_hess,
    #                       options={'xtol': 1e-4, 'disp': True},method='Newton-CG')                     
    #  if not result['success']:  
    #    bnds=[(0,None)]*pars.size
    #    result = op.minimize(loglik, result['x'], args=(ns,nt),jac=_llikjac,bounds=bnds,
    #                       options={'gtol': 1e-4, 'disp': True},method='TNC') 

    param = _par2covpar(result,ns,nt,covmodel) 
#  param = _par2covpar(result['x'],ns,nt,covmodel)   

                     
    return param#,result['success']    

def mlecovfitg(grid_s,grid_t,grid_v,covmodel,covparam0):
    '''
    Maximum likelihood method for grid format data
    
    opt_param=mlecovfit_v(ch,zh,covmodel,covparam)
    
    Input
    
    ch          n by d      array consisting of observation locations
    zh          n by 1      array containing obseved values
    covmodel    m           list of m nested covariance models in each of 
                            which the spatial and temporal components are put 
                            in a list as [covmodelS,covmodelT] 
    covparam    m           list of covariance parameters for m covmodels in 
                            each component the parameters are listed as 
                            [sill,[covparamS1,covparamS2,..],
                            [covparamT1,covparamT2,..]] 
    maxLagS     scalar      maximum spatial distance to be evaluated
    maxLagT     scalar      maximum temporal distance to be evaluated                         
    ''' 

    def loglik(pars,ch,ch_t,ns,nt,covmodelS,covmodelT):
        
        if any(pars<=0):
            llik=100000000000
            return llik
        
        covparamS,covparamT=_par2covSTpar(pars,ns,nt)

        # spatial part
        #covparamS = _par2covpar(par_S,ns,ns2,covmodelS)
        #Vs, V_list_s = coord2K(grid_s, grid_s, covmodelS, covparamS)
        Vs, V_list_s = coord2K(ch, ch, covmodelS, covparamS)
        # covariance tapering
        Vs[numpy.where(Vs/Vs[0,0]<10e-6)]=0
        Lv_s=numpy.linalg.cholesky(Vs)
        logdetV_s=2.*numpy.sum(numpy.log(numpy.diag(Lv_s)))
        #detV = numpy.linalg.det(V)
        n,m=grid_v.shape
        vs_m=grid_v.mean(1).reshape(n,1)
        vs_r=grid_v-vs_m.dot(numpy.ones((1,m)))
        S_s=vs_r.dot(vs_r.T)*1./(m-1)
        ddt_s=vs_m.dot(vs_m.T)
        term2_s=numpy.linalg.solve(Vs,S_s+ddt_s)
        llik_s=m*(logdetV_s+numpy.trace(term2_s))   

        # temporal part        
        #covparamT = _par2covpar(par_T,nt,nt2,covmodelT)
        #Vt, V_list_t = coord2K(grid_t, grid_t, covmodelT, covparamT)  
        Vt, V_list_t = coord2K(ch_t, ch_t, covmodelT, covparamT)
        Vt[numpy.where(Vt/Vt[0,0]<10e-6)]=0 # covariance tapering
        Lv_t=numpy.linalg.cholesky(Vt)
        logdetV_t=2.*numpy.sum(numpy.log(numpy.diag(Lv_t)))
        n,m=grid_v.shape
        vt_m=grid_v.mean(0).reshape(1,m)
        vt_r=grid_v-numpy.ones((n,1)).dot(vt_m)
        S_t=vt_r.T.dot(vt_r)*1./(n-1)
        ddt_t=vt_m.T.dot(vt_m)
        term2_t=numpy.linalg.solve(Vt,S_t+ddt_t)
        llik_t=m*(logdetV_t+numpy.trace(term2_t))        
        
        llik=llik_s+llik_t
        
        return llik    
         
    isST, isSTsep, modelS, modelT = isspacetime(covmodel)
    covmodelS=[]
    covmodelT=[]
    covparamS0=[]
    covparamT0=[]
    if isST:
        if isSTsep:       
            for model_s, model_t, param_i in zip( modelS,modelT,covparam0):
                if len(param_i)==1:
                    if model_s=='nuggetC' or model_s=='nuggetC':
                        sill=param_i
                        param_s=[None]
                        param_t=[None]
                    else:
                        print 'covparam is not consisent with covmodel'
                        raise 
                else:
                    sill, param_s, param_t = param_i                
                    covmodelS.append(model_s)
                    covmodelT.append(model_t)
                    covparamS0.append([sill,param_s])
                    covparamT0.append([sill,param_t])

        else:
            print 'S/T separability is currently assumed in MLE'
    else:
        print 'Pure spatial or temporal cases should use mlecovfitv'       
    
#    par_S,ns,ns2=_covpar2par(covparamS0)
#    par_T,nt,nt2=_covpar2par(covparamT0) 
#    numS=len(par_S)
#    numT=len(par_T)
#    pars = numpy.hstack([par_S,par_T])
    pars,ns,nt=_covpar2par(covparam0)
    
    ch=grid_s
    if type(grid_t[0])==numpy.datetime64:
        ch_t=numpy.double(numpy.asarray(grid_t,dtype='datetime64'))
    else:
        ch_t=grid_t
    ch_t=ch_t.reshape(len(ch_t),1)    

    low_bnd=numpy.ones(pars.size)*numpy.finfo(float).eps
    up_bnd=numpy.array([k for k in pars*3])
    args=[ch,ch_t,ns,nt,covmodelS,covmodelT]
    result, opt_val=bobyqa.bobyqa(loglik, pars, args, low_bnd, up_bnd )

    param = _par2covpar(result,ns,nt,covmodel) 

    return param
    
#    # Assure the following parameters have proper dimension or format, e.g., 
#    # 1D numpy array 
#    if grid_t is None:
#        grid_t=numpy.array([0]).reshape(1,1)
#  
#    if len(grid_s.shape)<2:
#        grid_s=numpy.reshape(grid_s,(grid_s.size,1))
#    
#    if maxLagS is None:
#        maxLagS=numpy.max(pdist(grid_s))
#    if maxLagT is None:
#        maxLagT=numpy.max(pdist(grid_t.reshape(grid_t.size,1)))
#        
#        
#    grid_t=numpy.asarray(grid_t)
#    grid_t=numpy.reshape(grid_t,(grid_t.size,1))
#    grid_v=grid_v.reshape((grid_s.shape[0],grid_t.size))
#
#    if grid_s.shape[0]<8000:
#        s_diff_i_left,s_diff_i_right,s_diff_v, _=diffarray(grid_s)
#    else:
#        s_diff_i_left,s_diff_i_right,s_diff_v, _= \
#          diffarray(grid_s,maxLagS)
#    nd=grid_s.shape[1]
#  
#    if nd==1:
#        s_diff_v=numpy.abs(s_diff_v).ravel()
#    elif nd==2:           
#        s_diff_v=numpy.sqrt(s_diff_v[:,0]**2+s_diff_v[:,1]**2)
#    elif nd==3:
#        s_diff_v=numpy.sqrt(s_diff_v[:,0]**2+s_diff_v[:,1]**2+s_diff_v[:,2]**2)
#  
#    if len(grid_t)<8000:
#        t_diff_i_left,t_diff_i_right,t_diff_v,_=diffarray(grid_t)
#    else:
#        t_diff_i_left,t_diff_i_right,t_diff_v,_= \
#            diffarray(grid_t,maxLagT)
#    t_diff_v=numpy.abs(t_diff_v).ravel()    
#    
#    idxs=numpy.where(s_diff_v<=maxLagS)
#    s_diff_i_left=s_diff_i_left[idxs].astype(numpy.int)
#    s_diff_i_right=s_diff_i_right[idxs].astype(numpy.int)
#    s_diff_v=s_diff_v[idxs]
#  
#    idxt=numpy.where(t_diff_v<=maxLagT)
#    t_diff_i_left=t_diff_i_left[idxt].astype(numpy.int)
#    t_diff_i_right=t_diff_i_right[idxt].astype(numpy.int)
#    t_diff_v=t_diff_v[idxt]      
#
#    lagS=numpy.unique(s_diff_v)
#    lagT=numpy.unique(t_diff_v)
    
    
    
    #for s in lagS:
        
    
def mlecovfit_sub(ch,zh,covmodel,covparam0):
    '''
    
    ch      n by d
    zh      n by m
    
    '''
    
    def loglik(pars,ns,nt):
        
        if any(pars <= 0):
            llik = 100000000000
            return llik
        
        covparam = _par2covpar(pars, ns, nt, covmodel)
        V, V_list = coord2K(ch, ch, covmodel, covparam)
        # covariance tapering
        V[numpy.where(V/V[0,0]<10e-6)]=0
        Lv=numpy.linalg.cholesky(V)
        logdetV=2.*numpy.sum(numpy.log(numpy.diag(Lv)))
        #detV = numpy.linalg.det(V)
        n,m=zh.shape
        zh_m=zh.mean(1).reshape(n,1)
        zh_r=zh-zh_m.dot(numpy.ones((1,m)))
        S=zh_r.dot(zh_r.T)*1./(m-1)
        ddt=zh_m.dot(zh_m.T)
        term2=numpy.linalg.solve(V,S+ddt)
        llik=m*(logdetV+numpy.trace(term2))
        return llik
        
    pars, ns, nt =_covpar2par(covparam0)
    if len(ch.shape) < 2:
        ch = ch.reshape(ch.size, 1)
    if type(ch[0,-1]) ==n umpy.datetime64:
        ch[:,-1] = numpy.double(numpy.asarray(ch[:,-1],d type='datetime64'))
        ch = ch.astype(numpy.double)

    low_bnd = numpy.ones(pars.size)*numpy.finfo(float).eps
    up_bnd = numpy.array([k for k in pars*3])
    args = [ns, nt]
    result, opt_val = bobyqa.bobyqa(loglik, pars, args, low_bnd, up_bnd )

    param = _par2covpar(result, ns, nt, covmodel) 

    return param#,result['success']     



# -*- coding: utf-8 -*-
"""
Created on Tue Nov 17 10:37:12 2015

@author: hdragon689
"""
import numpy
from ..general.coord2K import coord2K
from ..stats.anisotropy import aniso2iso

def simuchol(ch,covmodel,covparam,ns=1):
  '''
% simuchol                  - simulation by the Cholesky method
%
% Implementation of the traditional non conditional
% simulation method based on a Choleski decomposition
% of the covariance matrix. This simulation method is
% especially recommended for simulating independantly
% several sets of a limited number of hard values
% (less than few hundreds). Simulated values are zero
% mean Gaussian distributed. 
%
% SYNTAX :
%
% [Zh,L]=simuchol(ch,covmodel,covparam,ns);
%
% INPUT :
%
% ch         nh by d   matrix of coordinates for the locations
%                      where hard data have to be simulated. A
%                      line corresponds to the vector of coordinates
%                      at a simulation location, so the number of
%                      columns corresponds to the dimension of the
%                      space. There is no restriction on the dimension
%                      of the space.
% covmodel   string    string that contains the name of the covariance
%                      model which is used for the simulation (see the
%                      MODELS directory). Variogram models are not
%                      available for this function.
% covparam   1 by k    vector of values for the parameters of covmodel,
%                      according to the convention for the corresponding
%                      covariance model.
% ns         scalar    number of sets of simulated values which are required.
%                      If the optional ns variable is omitted from the input
%                      list of variables, simuchol.m produces a single set
%                      of simulated values.
% 
% OUTPUT :
%
% Zh         nh by ns  matrix of zero mean simulated Gaussian distributed hard
%                      values at the coordinates specified in ch. Each column
%                      corresponds to a different simulation, so that if ns=1
%                      or if ns is omitted, Zh is a column vector of values.
%                      Each simulated vector of values is statistically
%                      independant from the others.
% L          nh by nh  upper triangular matrix obtained from the Choleski
%                      decomposition of the global covariance matrix C for
%                      values at the ch coordinates, such that C=L'*L.

  '''  

#rand('state',sum(100*clock));
  K,_=coord2K(ch,ch,covmodel,covparam)
  L=numpy.linalg.cholesky(K)
#  Lt=L.T
#  n=Lt.shape[0]
  n=K.shape[0]
  Zh=numpy.zeros([n,ns])
  for i in xrange(ns): 
    Zh[:,i:i+1]=L.dot(numpy.random.randn(n,1))
  
  return Zh  
    
def anisosimuchol(ch,covmodel,covparam,theta,ratio,ns=1):
  newch=aniso2iso(ch,theta,ratio)
  K,_=coord2K(newch,newch,covmodel,covparam)
  L=numpy.linalg.cholesky(K)
#  Lt=L.T
#  n=Lt.shape[0]
  n=K.shape[0]
  Zh=numpy.zeros([n,ns])
  for i in xrange(ns): 
    Zh[:,i:i+1]=L.dot(numpy.random.randn(n,1))
    
  return Zh  

    
if __name__ == "__main__":  
  
  import matplotlib.pyplot as plt
  
  x=numpy.linspace(0,50,50)
  xi,yi=numpy.meshgrid(x,x)
  ch=numpy.hstack([xi.reshape(xi.size,1),yi.reshape(xi.size,1)])
#  covmodelS=['nuggetC','exponentialC','exponentialC']
#  covparamS=[[1,[None]],[0.45,[500]],[0.4,[300000]]]

  covmodelS=['exponentialC']
  covparamS=[[10,[20]]]  
  Z=simuchol(ch,covmodelS,covparamS)
  ZZ1=Z.reshape(xi.shape)
  
  plt.figure()
  plt.pcolor(xi,yi,ZZ1)
  plt.title('Large scale')
  plt.colorbar()
  plt.show()
  
  Za=anisosimuchol(ch,covmodelS,covparamS,theta=30,ratio=0.5)
  ZZ1a=Za.reshape(xi.shape)  
  
  plt.figure()
  plt.pcolor(xi,yi,ZZ1a)
  plt.title('Ansiotropic Large scale')
  plt.colorbar()
  plt.show()  
  
#  covmodelS=['exponentialC']
#  covparamS=[[10,[10]]]  
#  Z=simuchol(ch,covmodelS,covparamS)
#  ZZ2=Z.reshape(xi.shape)
#  
#  plt.figure()
#  plt.pcolor(xi,yi,ZZ2)
#  plt.title('Small scale')
#  plt.colorbar()
#  plt.show()
#  
#  plt.figure()
#  plt.pcolor(xi,yi,ZZ1+ZZ2)
#  plt.title('True')
#  plt.colorbar()
#  plt.show()  
#    


# -*- coding: utf-8 -*-
import numpy as np
import six


def get_standard_soft_pdf_type(pdf_type):
    ''' A function to define the softdata type in BME functions
    softpdf types can be specified by string or integer respectively

    softpdftype     PDF types                 associated parameters
    --------------------------------------------------------------------
    1               histogram                 (nl, limiB, probadens)
    2               linear                    (nl, limi, probadens)
    10              Gaussian or normal        (mean, var)
    
    More details: the way to define the flexible soft information follows 
    the definition of BMElib in Matlab as follows
    For a case of uniform distribution. For the histogram and linear-based 
    softdata can be expressed as below
    
    Histogram: softpdftype=1;nl=2;limi=[0,1];  probdens=[1]; 
    Linear: softpdftype=2;nl=2;limi=[0,1];  probdens=[1,1]
    
    '''
    if isinstance(pdf_type, six.string_types):
        if 'histogram'.startswith(pdf_type.lower()):
            return 1
        elif 'linear'.startswith(pdf_type.lower()):
            return 2
        elif 'gaussian'.startswith(pdf_type.lower()) or\
            'normal'.startswith(pdf_type.lower()):
            return 10
        else:
            raise ValueError('No supported pdftype found')
    else:
        if int(pdf_type) in [1]:
            return 1
        elif int(pdf_type) in [2] :
            return 2
        elif int(pdf_type) in [10] :
            return 10

def get_standard_order(order):
    '''
    A function to obtain the standard format of order parameter. The 
    order parameter determines the trend form in bme estimation.
    
    order  integer or    0 for constance mean and NaN for the zero mean   
           string        'zero mean' or 'constant mean' that explicitly describe 
                         the trend form
    
    Note: for S/T estimation, only zero and constant mean are supported for now 
          to avoid the overparametization of the trend modeling                                        
    
    '''
    ## For the string part, the string should be transformed captions into letters    
    ## before the comparison
    if isinstance(order, six.string_types): #base string
        if order.lower() == 'zero mean':
            return np.nan
        elif order.lower() == 'constant mean':
            return 0
        else:
            raise ValueError('No supported order (string) found')
    elif isinstance(order, int) or np.isnan(order): #int or np.nan
        if np.isnan(order):
            return order
        elif order in  [0, 0.0]:
            return 0
        else:
            raise ValueError('No supported order (number) found')
    else:
        print ('warning: "order" is not str or int, just return without modifing.')
        return order



# -*- coding: utf-8 -*-
# @Author: Chieh-Han Lee
# @Date:   2015-08-05 19:40:44
# @Last Modified by:   Chieh-Han Lee
# @Last Modified time: 2016-10-31 23:26:00
# -*- coding: utf-8 -*-
'''
Created on 2012/4/11

@author: KSJ
'''

import numpy as np

from scipy.spatial import cKDTree as KDTree
from scipy.spatial.distance import cdist as scipy_cdist

def idw_est( x, y, z, x_est, y_est ,power = 2):
    x, y, z, x_est, y_est =\
    map( lambda x : np.array( x, ndmin = 2 ),
         ( x, y, z, x_est, y_est ) )
    #dist_matrix = np.linalg.norm(
    #   np.hstack((x.T - x_est, y.T - y_est)) , axis=0 ) + 10**-10
    dist_matrix =\
        np.sqrt( ( x.T - x_est ) **2 + ( y.T - y_est ) **2 ) + 10**-10
    weight_matrix = np.reciprocal( dist_matrix ** power )
    up_matrix = weight_matrix * z.T
    up_matrix = up_matrix.sum( axis = 0 ) #sum column
    down_matrix = weight_matrix.sum( axis = 0 ) #sum column
    z_est = up_matrix / down_matrix
    return z_est

def idw_est_coord_value(coord, value, coord_est, power = 2):
    '''
    coord: a 2d array, r x d, row is data count, column is dimension
    value: a 2d array, r x 1, row is data count, column is value
    coord_est: dito coord
    '''
    coord_matrix = scipy_cdist(coord_est, coord) #coord_est by coord
    weight_matrix = np.reciprocal(coord_matrix**power)
    # remove dupliacted localtion (Set 0 wieght)
    weight_matrix[np.isinf(weight_matrix)] = 0.
    up_matrix = weight_matrix * value.T
    up_matrix = up_matrix.sum(axis=1, keepdims=True) #sum column
    down_matrix = weight_matrix.sum(axis=1, keepdims=True) #sum column
    value_est = up_matrix / down_matrix
    return value_est
 
def idw_kdtree( grid_s, grid_v, grid_s_est, nnear=10, eps=0, power=2, weights=None, leafsize=16 ):
    '''
    Inverse distance weighting (IDW) method using KDtree

    Syntax
        interp = idw_kdtree( grid_s, grid_v, grid_s_est, nnear=10, eps=0, power=2, weights=None, leafsize=10 ):
    
    Input
        grid_s:
            [r1 x d]. Coordinates in grid format.
        grid_v:
            [r1 x 1]. 
        grid_s_est:
            [r2 x d].
        nnear:
            integer. The list of k-th nearest neighbors to return. f k is an integer it is 
            treated as a list of [1, ... k] (range(1, k+1)). Note that the counting starts 
            from 1.
        eps:
            nonnegative float. Return approximate nearest neighbors;the k-th returned 
            value is guaranteed to be no further than (1+eps) times the distance to 
            the real k-th nearest neighbor.
        power:
            integer. Power parameter. Greater values of p assign greater influence to values 
            closest to the interpolated point, with the result turning into a mosaic of tiles 
            (a Voronoi diagram) with nearly constant interpolated value for large values of p
        weights:
            []. Weighted matrix.
        leafsize:
            positive integer. The number of points at which the algorithm switches over to brute-force.

    Output
        interp:
        [r2 x 1].Interpolation result of IDW.

    '''


    tree = KDTree(grid_s, leafsize=leafsize)

    distances, indices = tree.query(grid_s_est, k=nnear, eps=eps)
    interp = np.zeros( (len(grid_s_est),) + np.shape(grid_v[0]) )
    iternum = 0
    for dist, idx in zip(distances, indices):
        z0 = grid_v[idx[0]]
        if nnear == 1:
            weighted_v = grid_v[idx]
        elif dist[0] < 1e-10 and ~np.isnan(z0):
            weighted_v = grid_v[idx[0]]
        else:
            ix = np.where(dist==0)[0]
            if ix.size:
                dist = np.delete(dist, ix)
                idx = np.delete(idx, ix)
            ix = np.where(np.isnan(grid_v[idx]))[0]
            dist = np.delete(dist, ix)
            idx = np.delete(idx, ix)

            weight_matrix = np.reciprocal( dist ** power )
            if weights is not None:
                weight_matrix *= weights[idx]

            weight_matrix /= np.sum(weight_matrix)
            weighted_v = np.dot(weight_matrix, grid_v[idx])

        interp[iternum] = weighted_v
        iternum += 1

    return interp


if __name__ == "__main__":
    x = np.random.random(5)
    y = np.random.random(5)
    z = np.random.random(5)
    
    x_est = np.random.random(7)
    y_est = np.random.random(7)
    
    print idw_est( x, y, z, x_est, y_est)

    grid_s = np.random.random((100,2))
    grid_v = np.random.random((100,1))
    grid_s_est = np.random.random((7000,2))
    print idw_kdtree( grid_s, grid_v, grid_s_est )

    


import matplotlib.pyplot as plt
import numpy as np
import matplotlib.mlab as mlab
import math

def plot_marginal(m, v, zs=None, i=0, show=True):
    mu = m[i]
    variance = v[i]
    sigma = math.sqrt(variance)
    x = np.linspace(mu - 3*sigma, mu + 3*sigma, 100)
    plt.plot(x, mlab.normpdf(x, mu, sigma), label='general marginal pdf')

    if zs:
        zs_i = zs[i]
        mu = zs_i[1]
        variance = zs_i[2]
        sigma = math.sqrt(variance)
        x = np.linspace(mu - 3*sigma, mu + 3*sigma, 100)
        plt.plot(x, mlab.normpdf(x, mu, sigma), label='specific soft pdf')
    plt.title('index={i}'.format(i=str(i)))
    plt.legend()
    if show:
        plt.show()
    return plt


# -*- coding: utf-8 -*-
"""
Created on Fri Jul  3 09:46:39 2015

@author: hdragon689
"""
import numpy
import matplotlib.pyplot as plt
import matplotlib.dates as dates
import pandas
from pandas.tools.plotting import scatter_matrix


def colorplot(c,z,ax=None,cmap='hot_r',zrange=None,colorbar=True):  
  '''
  Plot colored symbols for the values of a vector at a set of two dimensional
  coordinates. The function uses a colormap such that the color displayed at
  these coordinates is a function of the corresponding values.  
  
  Syntax:
  ax=colorplot(c,z,ax=None,cmap='hot_r',zrange=None)
  
  Input:
  c         n by 2    2D numpy array of spatial coordinates
  z         n by 1    2D numpy array of observations
  ax        ax        optional. Axes object for colorplot. Default is None that 
                      creates new plot
  cmap      string    the color scheme. default is the reverse of hot scheme. 
                      Popular scheme are jet, hot, gray and their reverse xxx_r
                      the complete color scheme can refer to 
                      http://matplotlib.org/examples/color/colormaps_reference.html
  zrange    list      two scalar contains min and max of color range, [zmin, zmax]                
  colorbar  bool      default is True to display the colorbar    
  
  '''
  if ax is None:
    ax=plt.figure().add_subplot(111)
  else:
    ax=ax
    
  z=z.reshape(z.size,1)  
  if zrange is not None:
    plt.scatter(x=c[:,0],y=c[:,1],c=z,s=100, cmap=cmap, \
                  vmin=zrange[0], vmax=zrange[1])
  else:
    plt.scatter(x=c[:,0],y=c[:,1],c=z,s=100, cmap=cmap)    
  if colorbar:        
    plt.colorbar(ax=ax)
  
  xmax=numpy.max(c[:,0]); xmin=numpy.min(c[:,0])
  ymax=numpy.max(c[:,1]); ymin=numpy.min(c[:,1])
  xpadding=0.05*(xmax-xmin)
  ypadding=0.05*(ymax-ymin)
  plt.xlim(xmin-xpadding,xmax+xpadding)  
  plt.ylim(ymin-ypadding,ymax+ypadding)
  plt.show()
  plt.draw()
  
  return ax  

def markerplot(c,z,ax=None,symsize=300,zrange=None): 
  '''
  Plot the values of a vector at a set of two dimensional coordinates
  using symbols of varying sizes such that the size of the displayed
  symbols at these coordinates is a function of the corresponding values. 
  
  Syntax:
  ax=markerplot(c,z,ax=None,simsize=300,zrange=None)
  
  Input:
  c       n by 2    2D numpy array of spatial coordinates
  z       n by 1    2D numpy array of observations
  ax      ax        optional. Axes object for markerplot. Default is None that 
                    creates new plot
  symsize scalar    the size scheme. The size scale for marker display. 
                    Default is 300  
  zrange list       two scalar contains min and max of marker size range, [zmin, zmax]                
  
  '''  
  
  if ax is None:
    ax=plt.figure().add_subplot(111)
  else:
    ax=ax
    
  z=z.reshape(z.size,1)  
  zmax=numpy.max(z); zmin=numpy.min(z)
  z=z/(zmax-zmin)
  if zrange is not None:
    plt.scatter(x=c[:,0],y=c[:,1],s=z*symsize,vmin=zrange[0], vmax=zrange[1])
  else:
    plt.scatter(x=c[:,0],y=c[:,1], s=z*symsize)    

  xmax=numpy.max(c[:,0]); xmin=numpy.min(c[:,0])
  ymax=numpy.max(c[:,1]); ymin=numpy.min(c[:,1])
  xpadding=0.05*(xmax-xmin)
  ypadding=0.05*(ymax-ymin)
  plt.xlim(xmin-xpadding,xmax+xpadding)  
  plt.ylim(ymin-ypadding,ymax+ypadding)          
  plt.show()
  plt.draw()  
  
  return ax
  
  
def tsplot(t,z,ax=None,fmt='b-',displayscale=None):
  ''' 
  Plot time series data
  Syntax:
    tsplot(t,z,ax=None,fmt='b-',displayscale=None)
  
  Input:
  t             1 by n    1D or 2D array of time in datetime or 
                          numpy.datetime64 formats
  z             1 by n    1D or 2D array of observations
  ax            axes      the axes object to be plotted
  fmt           string    line format for time series plot. 
                          Details can refer to plt.plot?    
  displayscale  string    'm' and 'y' refers to the pre-specified xtick formats
                          for monthly or yearly data plotting. If None, the 
                          plot goes with the default in plt.plot_date function
                          
  Remark: Details of time xticker format can refer to 
          http://linux.die.net/man/3/strftime    
  '''
  
  if ax is None:  
    ax = plt.figure().add_subplot(111)
  else:
    ax=ax
  
  ax.plot_date(t,z,fmt=fmt)
  if displayscale=='m':
  #  xlocator=dates.YearLocator(1,month=7)
    xlocator=dates.MonthLocator([1,7],interval=1)
    months = dates.MonthLocator()  # every month
    ax.xaxis.set_major_locator(xlocator)
  #  ax.xaxis.set_minor_locator(x2locator)
    ax.xaxis.set_major_formatter(dates.DateFormatter('%b\n%Y')) 
    ax.xaxis.set_minor_locator(months)
  #  ax.xaxis.set_minor_formatter(dates.DateFormatter('%b\n%Y'))
  # Format can re
  elif displayscale=='y':  
    years = dates.YearLocator()   # every year
    months = dates.MonthLocator()  # every month
    yearsFmt = dates.DateFormatter('%Y')    
    ax.xaxis.set_major_locator(years)
    ax.xaxis.set_major_formatter(yearsFmt)
    ax.xaxis.set_minor_locator(months)  
    
  ax.autoscale_view()
  
  return ax

def histscatterplot(data,columns=None):
  '''
  Input
  data      m by n    a 2D array with m observations and n variables. This input
                      can also be a pandas.DataFrame 
  columns   list      list of the titles of variables
  
  Note: this function is an interface to the scatter_matrix function in pandas
  '''  
  
  
  if type(data) is not pandas.core.frame.DataFrame:
    data=pandas.DataFrame(data,columns=columns)    
  scatter_matrix(data,diagonal='hist',hist_kwds={'normed':'True'},alpha=0.2)    
  plt.show()
  plt.draw()    
  


if __name__ == "__main__":
  c=numpy.random.rand(30,2)
  z=numpy.random.rand(30,1)
  zr=[0,0.5]
  colorplot(c,z,ax=None,cmap='jet_r',zrange=zr)
  markerplot(c,z,symsize=300,zrange=zr)
  



# -*- coding: utf-8 -*-
"""
Created on Tue May 26 18:57:18 2015

@author: hdragon689
"""
import numpy as np
import warnings
import scipy.sparse.linalg as ssl  

from scipy.linalg import svd, diagsvd
try:
    from sklearn.decomposition import FastICA 
    from sklearn import mixture
except Exception, e:
    print 'warning: some functions cannot be used because:', e


def eof(X, n='all', norm=0, norms_direc=0, method='svd'):

    '''
    EOF - computes EOF of a matrix.
    
    Usage: [L, lambda, PC, EOFs, EC, error, norms] = EOF( M, num, norm, ... )
    
    Input: 
    M          m by n          the matrix with m obs and n items on which to  
                                                         perform the EOF.  
    num        scalar/string   the number of EOFs to return.  If num='all'(default), 
                                                         then all EOFs are returned.  
    norm       bool            normlaization flag. If it is true, then all time series 
                                                         are normalized by their standard deviation before EOFs 
                                                         are computed.  Default is false.  In this case,the 7-th 
                                                         output argument will be the standard deviations of each column.
    norm_direc integer         to designate the column==0 or row==1 to be normalized 
    method     string          method to be used for EOF calculation. svd or eig.
                                                         svd refers to singular value decomposition (default)
                                                         and eig refers to eigenvalue decomposition. In the
                                                         case of eig method, M should be a square matrix of 
                                                         the covariance functions to be assessed
    
    Output: 
    
    L       1 by k          1D array for the eigenvalues of the covariance matrix 
                                                    ( i.e. they are normalized by 1/(m-1), where m is the 
                                                    number of rows ).  
    lambdaU k by k          2D array for singular values (k by n for full matrix)
    PC      m by k          unitary matrix having left singular vectors as columns
    EOFs    n by k          principal components. unitary matrix haveing right 
                                                    sigular vectors as rows
    EC      m by k          expansion coefficients
    error   1 by k          1D array of the reconstruction error (L2-norm) for 
                                                    each item
    norms   1 by n          1D array for standard deviation of each item or one                    
    coefficients (PCs in other terminology) and error is the reconstruction
    error (L2-norm).
    
    Remark:
    Data is not detrended before handling.  If needed, perform the detrending before 
    using this function to fix that.
    This code is modified from the Matlab code by David M. Kaplan in 2003 by 
    Hwa-Lung Yu 2015/5/31
    
    '''  

    def rSVD(A, r, t=0, direction='default', Omega=None, n_rand=1, multi=False, queue=None):
        '''
        %-----------------------------------------------
        % Randomized truncated SVD
        % Input:
        %   A: m*n matrix to decompose (Need to be a numpy array)
        %   r: the number of singular values to keep (rank of S)
        %   t (optional): the number used in power (default = 0)
        %   direction (optional): to sample columns (=0) or sample rows (=1)
        %                         (default = 0, if m <= n
        %                                  = 1, if m > n )
        %   Omega (optional): the projection matrix to use in sampling
        %                     (program will compute A * Omega for direction = 0
        %                                           Omega * A for direction = 1)
        %
        % Output: classical output as the builtin svd matlab function
        %-----------------------------------------------
        '''

        m, n = A.shape
        U = np.zeros((m, r, n_rand))
        V = np.zeros((n, r, n_rand))
        S = np.zeros((r, r, n_rand))
        
        for k in range(n_rand):

            if direction == 'default':
                if m <= n:
                    direction = 0
                else:
                    direction = 1

            if direction == 0:
                Omega = np.random.randn(n, r)/np.sqrt(n)
            else:
                Omega = np.random.randn(m, r)/np.sqrt(m)

            if direction == 1:
                A = A.T
                Omega = Omega.T

            # Compute SVD
            if t == 0:
                Y = np.dot(A, Omega)
            else:
                Y = np.dot(np.dot(A, A.T)**t, np.dot(A, Omega))
            Q, _ = linalg.qr(Y, mode='economic')
            B = np.dot(Q.T, A)
            U_tild, S_temp, Vh_tild = linalg.svd(B, full_matrices=0)

            if direction == 0:
                U_temp = np.dot(Q, U_tild)
                Vh_temp = Vh_tild
            else:
                Vh_temp = np.dot(Q, U_tild)
                U_temp = Vh_tild

            V_temp = Vh_temp.T
            S_temp = linalg.diagsvd(S_temp, r, r)
            U[:, :, k] = U_temp[:, :r]
            S[:, :, k] = S_temp[:r, :r]
            V[:, :, k] = V_temp[:, :r]

        if multi:
            queue.put([U, S, V])
        else:
            return U, S, V

    def combine_orth(W_0, U_tild, tol_t=1e-2, tol_F=1e-4, iter_max=100):
        '''
        %-----------------------------------------------
        % Combine U_i to W
        % Input:
        %   W_0: the initial matrix used in updating. (Maybe U_0 is good.)
        %   U_tild: U_tild(:, :, i) = U_i
        %   tol_t(optional): stepsize < tol_t will stop. (default = 1e-2)
        %   tol_F(optional): 0 < (F_new -F_old)/F_old < tol_F will stop (default = 1e-3)
        %   iter_max(optional): max number of iteration.(default = 100)
        %
        % Output:
        %   W: the combined result
        %   n_iter: number of iterations
        %   W_record: W_record(:, :, i) = i_iter-th combined result
        %-----------------------------------------------
        '''
        
        m, r, n = U_tild.shape
        U_tild = np.reshape(U_tild, (m, n*r))
        W_record = np.zeros((m, r, iter_max))

        W = W_0
        GF = np.dot(U_tild, np.dot(U_tild.T, W))/n
        n_iter = 0
        F_old = np.trace(np.dot(GF.T, W))
        t = 1
        
        while n_iter < iter_max:
            # Calculate the gradient
            U = np.hstack([GF, W])
            V = np.hstack([W, -GF])
            Y = W + t*(GF - np.dot(W, np.dot(W.T, GF)))/2
            X = np.dot(np.linalg.inv(-np.eye(2*r) + t*np.dot(V.T, U))/2, np.dot(V.T, Y))
            #  Updating
            W_new = Y - t*np.dot(U, X)/2
            
            F_new = np.trace(np.dot(np.dot(W_new.T, U_tild), np.dot(U_tild.T, W_new))/n)

            while F_new < F_old:
                t = t/2
                if t < tol_t:
                    print 'The step size is too small.'
                    W_record[:, :, n_iter+1:-1] = []
                    break
                Y = W + t/2*(GF - np.dot(W, np.dot(W.T, GF)))
                X = np.dot(np.linalg.inv(np.eye(2*r) + t/2*np.dot(V.T, U)), np.dot(V.T, Y))
                W_new = Y - t/2*np.dot(U, X)

                F_new = np.trace(np.dot(np.dot(W_new.T, U_tild), np.dot(U_tild.T, W_new))/n)

            if (F_new-F_old)/F_old < tol_F:
                print "The change rate(%f) is too small." % ((F_new-F_old)/F_old)
                break
            else:
                W = W_new
                GF = np.dot(U_tild, np.dot(U_tild.T, W))/n
                F_old = F_new
                W_record[:, : , n_iter] = W
                n_iter += 1
                
        W_record = np.delete(W_record, np.s_[n_iter:iter_max], 2)

        return W, n_iter, W_record

    # EOF start

    s = X.shape
    ss = np.min(s)    
    
    # Normalized by standard deviation if desired

    if norm:
        norms = np.std(X,axis=norms_direc)  
    else:
        norms = np.ones(s[1])
                
    X = np.dot(X, np.diag(1/norms))

    # Using SVD to solve the eigen problem of covariance matrix
    
    if method=='svd':
        # To solve with full matrices
        if n=='all' or n>=ss:
            U, lambdaU, Vh = svd(X, full_matrices=True) 
            PCs = U
            EOFs = Vh.T
        else:
        # To solve with the first n singular vectors/values
            U, lambdaU, Vh = ssl.svds(X, n)
            lambdaU = lambdaU[::-1]
            PCs = U[:,::-1]
            EOFs = Vh[::-1,:].T
        
        L = lambdaU**2/(s[0]-1)

        # Check computation errors
        if n=='all':
            lambdaU2 = diagsvd(lambdaU, s[0], s[1])
            ECs = PCs.dot(lambdaU2)
            diff = X-np.dot(ECs, EOFs.T)
            error = np.sqrt(sum(sum(diff*np.conj(diff))))  
        else:
            lambdaU2 = np.diag(lambdaU)
            ECs = np.dot(PCs, lambdaU2)
            diff = X-np.dot(ECs,EOFs.T)
            error = np.sqrt(sum(sum(diff*np.conj(diff))))

    else:
        CU = 1./(s[0]-1)*X.T.dot(X)
        L, EOFs = np.linalg.eig(CU)
        if np.any(np.iscomplex(EOFs)):
            EOFs = EOFs.real
            L = L.real
        ECs = (EOFs.T.dot(X.T)).T
        lambdaU2 = np.sqrt(L)    
     # L= lambdaU2**2/(s[0]-1)
        lambdaU2[np.where(np.isnan(lambdaU2))] = 0.
        PC = ECs*1/lambdaU2 # the dimension of L should be checked
        lambdaU2 = np.diag(lambdaU2)
        diff = X-np.dot(ECs,EOFs.T)
        error = np.sqrt(sum(sum(diff*np.conj(diff)))) 

    return L, lambdaU2, PCs, EOFs, ECs, error, norms

''' Extended EOF '''

def eeof(M, tl, n='all'):
    '''
    Extended EOF - computes EOF of a matrix.
    
    Usage: [L, lambda, PC, EOFs, EC, error, norms] = EEOF( M, num, norm, ... )
    
    Input: 
    M       n by p        the matrix with n obs and p items on which to perform 
                                                the EOF.  
    tl      integer       Window size to assess the time lags. In this case, the 
                                                space-time dynamic pattern of size of tl by p is assessed                 

    Output:
    EEOF    list          the n-tlag principal extended EOFs. Each has size of 
                                                tlag by p. For the space-time case, p is the number of 
                                                stations
    EEC     n-tl by n-tl  Each column contains the extended EC for the corresponding
                                                extended EOF 
    lam     1 by n-tl     1D array of the eigenvalues of the corresponding n-tl EEOFs                  
 
    '''
    n,p=M.shape
    # XX=np.zeros((w,p*(n-w)))
    XX=np.zeros((n-tl,p*tl))
    for i in xrange(n-tl):
        # M2=M[i:n-w+i,:]
        M2=M[i:tl+i,:]
        Mv=M2.reshape(1,M2.size,order='F')
        XX[i,:]=Mv
 
 # lam, sigv, PC, EOFs, EC, diff, error=sreof(XX)
    EOFs,EC,sigv,lam=sreof(XX)
    
    # to create the high-dimensional matrix with consistant format of M
    EEOF=[]
    for i in xrange(EOFs.shape[1]):
        EEOF.append(EOFs[:,i].reshape(tl,p))

    EEC=EC
    
    return EEOF, EEC, lam

'''
def eeofplot(EEOF,EEC): 
    '''

        
    

'''
def bveof(X,Y)
to estimate the bivariate EOFs
'''
     
''' varimax function'''   
def varimax_(Phi, gamma = 1.0, q = 20, tol = 1e-6):
    '''
    This code is obtained from
    https://en.wikipedia.org/wiki/Talk%3aVarimax_rotation
    '''  
#  from numpy import eye, asarray, dot, sum
#  from numpy.linalg import svd
    p, k = Phi.shape
    R = np.eye(k)
    d=0
    for i in xrange(q):
        d_old = d
        Lambda = np.dot(Phi, R)
        u, s, vh = svd(np.dot(Phi.T, Lambda**3 - (gamma/p) * Lambda.dot(np.diag(np.diag(Lambda.T.dot(Lambda))))))
        R = np.dot(u,vh)
        d = np.sum(s)
        if d_old!=0 and d/d_old < 1 + tol: break
    return np.dot(Phi, R), R


def sreof( U, m='all', norm=0, *args ):
    '''
    Scaled and rotaed EOF - computes scaled and rotated EOF of a matrix. 
    
    Usage: [EOF,EC,lam,eigval] = SREOF( M, num, norm, ... )
    
    Input: 
    M     m by n          the 2D array with m obs and n items on which to perform 
                                                the EOF.  
    num   scalar/string   the number of EOFs to return.  If num='all'(default), 
                                                then all EOFs are returned.  
    norm  bool            normlaization flag. If it is true, then all time series 
                                                are normalized by their standard deviation before EOFs 
                                                are computed.  Default is false.  In this case,the 7-th 
                                                output argument will be the standard deviations of each column.
    Others 
    ... are extra arguments to be given to the svds function.  These will
    be ignored in the case that all EOFs are to be returned, in which case
    the svd function is used instead. Use these with care. 
    Try scipy.sparse.linalg.svds?
    
    Output: 
    
    EOF     m by num        the EOF results are listed in the columns  
    EC      n by num        the correponding EC results are listed in the columns
    lam     1 by num        1D array of the singular values the EOFs
    eigval  1 by num        1D array of the eigenvalues of the EOFs

    
    Remark:
    The result is re-scaled from the original EOF results by considering the 
    eigenvalues of each EOFs. The rotation is performed on the scaled EOFs.
    By doing this, the EOF rotation can have relatively low impacts from the number
    of EOFs to be rotated and therefore the results can be stabilized. The 
    rotation is based upon the 

    '''  

    L,lambdaU,PC,EOFs,EC,error,norms=eof(U,m,norm,*args)
    
    SEOF = EOFs.dot(lambdaU.T[:,:EOFs.shape[1]]) # scaled EOFs by singular values
    
    # rotate the EOFs by using varimax method  
#  rotm,REOF,__= varimax(SEOF) # this is a slower varimax
#  REOF=np.dot(EOFs,rotm)
    REOF, rotm = varimax_(SEOF, gamma = 1.0, q = 50, tol = 1e-7)
    REOF = np.dot(EOFs[:,:rotm.shape[0]],rotm)
    # rotate the corresponding ECs
    NewEOFs=REOF
    NewEC=np.dot(EC[:,:rotm.shape[0]],np.linalg.inv(rotm.T))
#  NewEC=np.dot(EC,np.linalg.inv(rotm.T))

    
    if (type(m) is str) or (m is 'all'):
        m = rotm.shape[0]

    for i in xrange(m):
        maxi = np.where((np.abs(NewEOFs[:,i]).max()==np.abs(NewEOFs[:,i])))
        signi = np.float(NewEOFs[:,i][maxi]/np.abs(NewEOFs[:,i][maxi]))
        NewEOFs[:,i] = signi*NewEOFs[:,i]
        NewEC[:,i] = signi*NewEC[:,i]
        
    return NewEOFs, NewEC, lambdaU, L


    
def srpca(U, m='all', norm=0, *args):
    '''
    Scaled and rotaed EC - computes scaled and rotated EC of a matrix. 
    
    Usage: [L, lambda, PC, EOFs, EC, error, norms] = SRPCA( M, num, norm, ... )
    
    Input: 
    M     m by n          the 2D array with m obs and n items on which to perform 
                                                the EOF.  
    num   scalar/string   the number of EOFs to return.  If num='all'(default), 
                                                then all EOFs are returned.  
    norm  bool            normlaization flag. If it is true, then all time series 
                                                are normalized by their standard deviation before EOFs 
                                                are computed.  Default is false.  In this case,the 7-th 
                                                output argument will be the standard deviations of each column.
    Others 
    ... are extra arguments to be given to the svds function.  These will
    be ignored in the case that all EOFs are to be returned, in which case
    the svd function is used instead. Use these with care. 
    Try scipy.sparse.linalg.svds?
    
    Output: 
    
    EOF   m by num        the EOF results are listed in the columns  
    EC    n by num        the correponding EC results are listed in the columns 

    
    Remark:
    The result is re-scaled from the original EOF results by considering the 
    eigenvalues of each EOFs. The rotation is performed on the scaled EOFs.
    By doing this, the EOF rotation can have relatively low impacts from the number
    of EOFs to be rotated and therefore the results can be stabilized. The 
    rotation is based upon the 

    '''  
        
    L,lambdaU,PC,EOFs,EC,error,norms=eof(U,m,norm,*args)

    SEC=EC*lambdaU # scaled EOFs by eigenvalues

    # rotate the EOFs by using varimax method  
    rotm,REC, eps= varimax(SEC)
    REC=np.dot(EC,rotm)

    # rotate the corresponding ECs
    NewEC=REC
    NewEOFs=np.dot(EOFs,np.linalg.inv(rotm.T))
    Newlambda=lambdaU
    
    for i in xrange(m):
        maxi=np.where((np.abs(NewEC[:,i]).max()==np.abs(NewEC[:,i])))
        signi=np.float(NewEC[:,i][maxi]/np.abs(NewEC[:,i][maxi]))
        NewEOFs[:,i]=signi*NewEOFs[:,i]
        NewEC[:,i]=signi*NewEC[:,i]
        
    return NewEOFs, NewEC, Newlambda


def varimax(amat,target_basis=None):
    '''
    [ROTM,OPT_AMAT] = varimax(AMAT,TARGET_BASIS)
    
    Gives a (unnormalized) VARIMAX-optimal rotation of matrix AMAT:
    The matrix  AMAT*ROTM  is optimal according to the VARIMAX
    criterion, among all matricies of the form  AMAT*R  for  
    R  an orthogonal matrix. Also (optionally) returns the rotated AMAT matrix
    OPT_AMAT = AMAT*ROTM.
    
    Uses the standard algorithm of Kaiser(1958), Psychometrika.
    
    Inputs:
    
    AMAT          N by K     matrix of "K component loadings"
    TARGET_BASIS  N by N     (optional) an N by N matrix whose columns 
                                                     represent a basis toward which the rotation will 
                                                     be oriented; the default is the identity matrix 
                                                     (the natural coordinate system); this basis need 
                                                     not be orthonormal, but if it isn't, it should be
                                                     used with great care!
    Outputs: 
    
    ROTM         K by K      Optimizing rotation matrix
    OPT_AMAT     N by K      Optimally rotated matrix  (AMAT*ROTM)
    
    Modified by Trevor Park in April 2002 from an original file by J.O. Ramsay  
    Modified by H-L Yu into python code in June 2015 
    '''
    MAX_ITER=50
    EPSILON=1e-7
    
    amatd=amat.shape
    
    if np.size(amatd) != 2:
        raise RuntimeError('AMAT must be two-dimensional')
        
    n=amatd[0]
    k=amatd[1]
    rotm=np.eye(k)
    
    if k==1:
        return
        
    if target_basis==None:
        target_basis_flag=0
        target_basis=np.eye(n)    
    else:
        target_basis_flag=1
        if np.size(target_basis.shape) !=2:
            raise RuntimeError('TARGET_BASIS must be two-dimensional')
        if target_basis.shape==(n,n):
            amat=np.dot(np.linalg.inv(target_basis),amat)
        else:
            raise RuntimeError('TARGET_BASIS must be a basis for the column space')
    
    varnow=np.sum(np.var(amat**2,0))
    not_converged=1
    iterx=0
    while not_converged and iterx < MAX_ITER:
        for j in xrange(k-1):
            for l in xrange(j+1,k):
                # Calculate optimal 2-D planar rotation angle for columns j,l
                phi_max=np.angle(n*np.sum(np.vectorize(complex)(amat[:,j],amat[:,l])**4) \
                                 - np.sum(np.vectorize(complex)(amat[:,j],amat[:,l])**2)**2)/4
                sub_rot = np.array([[np.cos(phi_max),-np.sin(phi_max)],\
                                            [np.sin(phi_max),np.cos(phi_max)]])
                amat[:,[j,l]]=np.dot(amat[:,[j,l]],sub_rot)
                rotm[:,[j,l]]=np.dot(rotm[:,[j,l]],sub_rot)   
                
        varold = varnow
        varnow = np.sum(np.var(amat**2,0))      
    
        if varnow==0:
            return 
        
        not_converged = ((varnow-varold)/varnow > EPSILON)
        iterx= iterx +1
        
    if iterx >= MAX_ITER:
        warnings.warn('Maximum number of iterations reached in function')
    
    if target_basis_flag:  
        opt_amat=target_basis*amat
    else:
        opt_amat=np.dot(amat,rotm)
        
    eps=(varnow-varold)/varnow  

    return rotm, opt_amat, eps  

def princomp(A):
        """ 
        coeff,score,latent=princomp(A)   
        
        This function performs principal components analysis (PCA) 
        on the n-by-p data matrix A. Rows of A correspond to observations, 
        columns to variables. 
        
        Input:
        
        A       n by p      matrix of n observations of p variables
         
        Output:  
        
        coeff   p by p      a p-by-p matrix, each column containing coefficients 
                                                for one principal component.
        score   n by p      the principal component scores; that is, the 
                                                representation of A in the principal component space. 
                                                Rows of SCORE correspond to observations, columns to 
                                                components.
        latent : 
                a vector containing the eigenvalues 
                of the covariance matrix of A.
                
         Ref: this function is downloaded from the link
         http://glowingpython.blogspot.tw/2011/07/principal-component-analysis-with-numpy.html
         """
        # computing eigenvalues and eigenvectors of covariance matrix
        M = (A-np.mean(A.T,axis=1)).T # subtract the mean (along columns)
        [latent,coeff] = np.linalg.eig(np.cov(M)) # attention:not always sorted
        score = np.dot(coeff.T,M) # projection of the data in the new space
        return coeff,score,latent

'''Independent component analysis'''

def stica(M, num='all', norm=0, norms_direc=0, ortho='s'):
    ''' ICA for S/T dataset that decomposes independent S/T signals from S/T 
            dataset  
    stica - computes ICAs of a matrix.
    
    Usage: [L, lambda, PC, EOFs, EC, error, norms] = stica( M, num, norm, ... )
    
    Input: 
    M          m by n          the matrix with m obs and n items on which to  
                                                         perform the ICA.  
    num        scalar/string   the number of ICAs to return.  If num='all'(default), 
                                                         then all ICAs are returned.  
    norm       bool            normlaization flag. If it is true, then all time series 
                                                         are normalized by their standard deviation before EOFs 
                                                         are computed.  Default is false.  In this case,the 7-th 
                                                         output argument will be the standard deviations of each column.
    norm_direc integer         to designate the column==0 or row==1 to be normalized 
    ortho      string          method to identify orthogonal vectors, 's' denotes
                                                         the spatial independent functions (default) 
                                                         and 't' denotes the temporal independent functions

    Output: 
    EOFs    n by k          Spatial functions which are independent while ortho is 
                                                    's'
    EC      m by k          Temporal functions which are independent while ortho is 
                                                    't'
    lambda  k by 1          1D array for singular values (k by n for full matrix) 
    
    Note:  Singular values are approximated by the norm of the mixing matrix of 
    the ICA. This should be further checked. 
                                                    
            
    ''' 

    m,n=M.shape
    # ss=np.min([m,n])      
    # Normalized by standard deviation if desired

    if norm:
        norms=np.std(M,axis=norms_direc)  
    else:
        norms=np.ones(n)
                
    M=np.dot(M, np.diag(1/norms))  
    
    if num is 'all':
        n_com=n
    else:
        n_com=num
    
    ica = FastICA(n_components=n_com)
    if ortho is 't':
        ICAs=ica.fit_transform(M)
        eofICA=ica.mixing_
        lambdaM=np.linalg.norm(eofICA,axis=0)
    else:
        eofICA=ica.fit_transform(M.T)
        ICAs=ica.mixing_
        lambdaM=np.linalg.norm(ICAs,axis=0)
    
    return eofICA,ICAs,lambdaM 
    
def srica(U, m='all', norm=0, ortho='s', *args):
    '''
    Scaled and rotaed ICA - computes scaled and rotated ICA of a matrix. 
    
    Usage: [EOF,EC,lam,eigval] = SREOF( M, num, norm, ... )
    
    Input: 
    M     m by n          the 2D array with m obs and n items on which to perform 
                                                the EOF.  
    num   scalar/string   the number of EOFs to return.  If num='all'(default), 
                                                then all EOFs are returned.  
    norm  bool            normlaization flag. If it is true, then all time series 
                                                are normalized by their standard deviation before EOFs 
                                                are computed.  Default is false.  In this case,the 7-th 
                                                output argument will be the standard deviations of each column.
    ortho      string     method to identify orthogonal vectors, 's' denotes
                                                the spatial independent functions (default) 
                                                and 't' denotes the temporal independent functions

    Output: 
    
    EOF     m by num        the EOF (spatial function) results are listed in the 
                                                    columns  
    EC      n by num        the correponding EC (temporal function) results are 
                                                    listed in the columns
    lam     1 by num        1D array of the singular values the EOFs/ECs which are 
                                                    with respect to 's' or 't'
    
    Remark:
    The result is re-scaled from the original EOF results by considering the 
    eigenvalues of each EOFs. The rotation is performed on the scaled EOFs.
    By doing this, the EOF rotation can have relatively low impacts from the number
    of EOFs to be rotated and therefore the results can be stabilized. The 
    rotation is based upon the 

    '''  
    
    EOFs,EC,lambdaU=stica(U, num=m,norm=norm,ortho=ortho)
    
    if ortho is 's': 
        SEOF=EOFs*lambdaU#EOFs.dot(np.diag(lambdaU)) # scaled EOFs by eigenvalues

        # rotate the EOFs by using varimax method  
        # rotm1,REOF1,__= varimax(SEOF) # this is a slower varimax
        REOF,rotm=varimax_(SEOF, gamma = 1.0, q = 50, tol = 1e-7)
        REOF=np.dot(EOFs[:,:rotm.shape[0]],rotm)
    
        # rotate the corresponding ECs
        NewEOFs=REOF
        NewEC=np.dot(EC[:,:rotm.shape[0]],np.linalg.inv(rotm.T))
    
        if (type(m) is str) or (m is 'all'):
            m=rotm.shape[0]

        for i in xrange(m):
            maxi=np.where((np.abs(NewEOFs[:,i]).max()==np.abs(NewEOFs[:,i])))
            signi=np.float(NewEOFs[:,i][maxi]/np.abs(NewEOFs[:,i][maxi]))
            NewEOFs[:,i]=signi*NewEOFs[:,i]
            NewEC[:,i]=signi*NewEC[:,i]
    else:
        SEC=EC*lambdaU # scaled EOFs by eigenvalues

        # rotate the EOFs by using varimax method  
        REC,rotm = varimax_(SEC, gamma = 1.0, q = 50, tol = 1e-7)
        REC=np.dot(EC,rotm) #np.dot(EC[:,:rotm.shape[0]],rotm)

        # rotate the corresponding ECs
        NewEC=REC
        NewEOFs=np.dot(EOFs,np.linalg.inv(rotm.T))
    
        for i in xrange(m):
            maxi=np.where((np.abs(NewEC[:,i]).max()==np.abs(NewEC[:,i])))
            signi=np.float(NewEC[:,i][maxi]/np.abs(NewEC[:,i][maxi]))
            NewEOFs[:,i]=signi*NewEOFs[:,i]
            NewEC[:,i]=signi*NewEC[:,i]    
        
    return NewEOFs,NewEC,lambdaU 

def eofclass(EOFs,num='all'):
    '''
    To classify the identified EOF locations.  
    
    EOFclass,means,covs,weights,logprob,prob_comps,clf=eofclass(EOFs,num='all')   
    
    Input:
    EOFs      m by n        the EOF (spatial function) results are listed in the 
                                                    columns
    num       scalar/string number of EOFs functions to be classified. Default is 
                                                    'all' that all  functions will be used                           
    
    Output:
    EOFclass  m by num      indicators for the identified regions. 1 denotes the 
                                                    identified locations; otherwise, 0 is shown 
    means     num by 2      the mean of EOF values of the two classified regions.
    covs      num by 2      the variances of EOF values of the two classified 
                                                    regions.
    bound     num by 1      a 1-D array denotes the boundary values to classify 
                                                    EOF values in the num of EOF functions.
    
    Note: The classification is performed by using the two-component Gaussian 
    mixture model to reveal the mean and variance of the close-to-zero EOF region.
    t-test is used to identify the locations which are significantly deviate from 
    zeros.
    '''  
    
    m,n=EOFs.shape
    
    if num is 'all':
        num=n
    
    EOFclass=np.zeros([m,num])
    means=np.zeros([num,2])
    covs=np.zeros([num,2])
    weights=np.zeros([num,2])
#  logprob=[None]*num
#  prob_comps=[None]*num
    clf=[None]*num
    bound=np.zeros(num)
    for i in xrange(num):
        clf[i]=mixture.GMM(n_components=2)
        clf[i].fit(EOFs[:,i:i+1])
        weights[i,0:2]=clf[i].weights_
        means[i,0:2]=clf[i].means_.reshape(2,)
        covs[i,0:2]=clf[i].covars_.reshape(2,)
        signs=means[i,0:2]/np.abs(means[i,0:2])
        popmean=np.min(np.abs(means[i,0:2]))
        idx=np.where(np.abs(means[i,0:2])==popmean)
        popmean=popmean*signs[idx]
        bound[i]=np.sqrt(covs[i,idx])*1.95996
        EOFclass[np.where(EOFs[:,i]>=bound[i]),i:i+1]=1
                
        
#    EOFclass[:,i:i+1]=clf[i].predict(EOFs[:,i:i+1]).reshape(EOFs[:,i].size,1)
#    logprob[i],prob_comps[i]=clf[i].score_samples(EOFs[:,i:i+1])
#    idxmax=np.where(EOFs[:,i]==EOFs[:,i].max())
#    if EOFclass[idxmax,i]==0:
#      EOFclass[np.where(EOFclass[:,i]==0),i]=2
#      EOFclass[np.where(EOFclass[:,i]==1),i]=0
#      EOFclass[np.where(EOFclass[:,i]==2),i]=1
#      weights[i,0],weights[i,1]=weights[i,1],weights[i,0]
#      means[i,0],means[i,1]=means[i,1],means[i,0]
#      covs[i,0],covs[i,1]=covs[i,1],covs[i,0]
#      prob_comps[i][:,[0,1]]=prob_comps[i][:,[1,0]]      
                
#  return EOFclass,means,covs,weights,logprob,prob_comps,clf  
    return EOFclass,means,covs,bound
    
## EOF classification
#EOF1=EOFdf.values[:,0:1]
#clf = mixture.GMM(n_components=2)
#clf.fit(EOF1)
#wei=clf.weights_
#mea=clf.means_
#cov=clf.covars_
#EOF1_classlabel=clf.predict(EOF1).reshape(EOF1.size,1) # preform classification on EOF1
#xxx=np.linspace(EOF1.min(),EOF1.max())
#xxx=xxx.reshape(xxx.size,1)
#plt.figure()
#plt.hist(EOF1,bins=20,normed=True)
#plt.plot(xxx,np.exp(clf.score_samples(xxx)[0]))
#plt.show()


'''
#!/usr/bin/env python
# -*- coding: ascii -*-

"""Higher order singular value decomposition routines

as introduced in:
        Lieven de Lathauwer, Bart de Moor, Joos Vandewalle,
        'A multilinear singular value decomposition',
        SIAM J. Matrix Anal. Appl. 21 (4), 2000, 1253-1278

implemented by Jiahao Chen <jiahao@mit.edu>, 2010-06-11

Disclaimer: this code may or may not work.
"""

__author__ = 'Jiahao Chen <jiahao@mit.edu>'
__copyright__ = 'Copyright (c) 2010 Jiahao Chen'
__license__ = 'Public domain'

try:
        import numpy as np
except ImportError:
        print "Error: HOSVD requires numpy"
        raise ImportError



def unfold(A,n):
        """Computes the unfolded matrix representation of a tensor

        Parameters
        ----------

        A : ndarray, shape (M_1, M_2, ..., M_N)

        n : (integer) axis along which to perform unfolding,
                                    starting from 1 for the first dimension

        Returns
        -------

        Au : ndarray, shape (M_n, M_(n+1)*M_(n+2)*...*M_N*M_1*M_2*...*M_(n-1))
                 The unfolded tensor as a matrix

        Raises
        ------
        ValueError
                if A is not an ndarray

        LinAlgError
                if axis n is not in the range 1:N

        Notes
        -----
        As defined in Definition 1 of:

                Lieven de Lathauwer, Bart de Moor, Joos Vandewalle,
                "A multilinear singular value decomposition",
                SIAM J. Matrix Anal. Appl. 21 (4), 2000, 1253-1278
        """

        if type(A) != type(np.zeros((1))):
                print "Error: Function designed to work with numpy ndarrays"
                raise ValueError
        
        if not (1 <= n <= A.ndim):
                print "Error: axis %d not in range 1:%d" % (n, A.ndim)
                raise np.linalg.LinAlgError

        s = A.shape

        m = 1
        for i in range(len(s)):
                m *= s[i]
        m /= s[n-1]

        #The unfolded matrix has shape (s[n-1],m)
        Au = np.zeros((s[n-1],m))

        index = [0]*len(s)

        for i in range(s[n-1]):
                index[n-1] = i
                for j in range(m):
                        Au[i,j] = A[tuple(index)]

                        #increment (n-1)th index first
                        index[n-2] += 1

                        #carry over: exploit python's automatic looparound of addressing!
                        for k in range(n-2,n-1-len(s),-1):
                                if index[k] == s[k]:
                                        index[k-1] += 1
                                        index[k] = 0

        return Au



def fold(Au, n, s):
        """Reconstructs a tensor given its unfolded matrix representation

        Parameters
        ----------

        Au : ndarray, shape (M_n, M_(n+1)*M_(n+2)*...*M_N*M_1*M_2*...*M_(n-1))
                 The unfolded matrix representation of a tensor

        n : (integer) axis along which to perform unfolding,
                                    starting from 1 for the first dimension

        s : (tuple of integers of length N) desired shape of resulting tensor

        Returns
        -------
        A : ndarray, shape (M_1, M_2, ..., M_N)

        Raises
        ------
        ValueError
                if A is not an ndarray

        LinAlgError
                if axis n is not in the range 1:N

        Notes
        -----
        Defined as the natural inverse of the unfolding operation as defined in Definition 1 of:

                Lieven de Lathauwer, Bart de Moor, Joos Vandewalle,
                "A multilinear singular value decomposition",
                SIAM J. Matrix Anal. Appl. 21 (4), 2000, 1253-1278
        """

        m = 1
        for i in range(len(s)):
                m *= s[i]
        m /= s[n-1]

        #check for shape compatibility
        if Au.shape != (s[n-1], m):
                print "Wrong shape: need", (s[n-1], m), "but have instead", Au.shape
                raise np.linalg.LinAlgError

        A = np.zeros(s)

        index = [0]*len(s)

        for i in range(s[n-1]):
                index[n-1] = i
                for j in range(m):
                        A[tuple(index)] = Au[i,j]

                        #increment (n-1)th index first
                        index[n-2] += 1

                        #carry over: exploit python's automatic looparound of addressing!
                        for k in range(n-2,n-1-len(s),-1):
                                if index[k] == s[k]:
                                        index[k-1] += 1
                                        index[k] = 0

        return A



def HOSVD(A):
        """Computes the higher order singular value decomposition of a tensor

        Parameters
        ----------

        A : ndarray, shape (M_1, M_2, ..., M_N)

        Returns
        -------
        U : list of N matrices, with the nth matrix having shape (M_n, M_n)
                The n-mode left singular matrices U^(n), n=1:N

        S : ndarray, shape (M_1, M_2, ..., M_N)
                The core tensor

        D : list of N lists, with the nth list having length M_n
                The n-mode singular values D^(n), n=1:N

        Raises
        ------
        ValueError
                if A is not an ndarray

        LinAlgError
                if axis n is not in the range 1:N

        Notes
        -----
        Returns the quantities in Equation 22 of:

                Lieven de Lathauwer, Bart de Moor, Joos Vandewalle,
                "A multilinear singular value decomposition",
                SIAM J. Matrix Anal. Appl. 21 (4), 2000, 1253-1278
        """

        Transforms = []
        NModeSingularValues = []

        #--- Compute the SVD of each possible unfolding
        for i in range(len(A.shape)):
                U,D,V = np.linalg.svd(unfold(A,i+1))
                Transforms.append(np.asmatrix(U))
                NModeSingularValues.append(D)

        #--- Compute the unfolded core tensor
        axis = 1 #An arbitrary choice, really
        Aun = unfold(A,axis)

        #--- Computes right hand side transformation matrix
        B = np.ones((1,))
        for i in range(axis-A.ndim,axis-1):
                B = np.kron(B, Transforms[i])

        #--- Compute the unfolded core tensor along the chosen axis
        Sun = Transforms[axis-1].transpose().conj() * Aun * B

        S = fold(Sun, axis, A.shape)

        return Transforms, S, NModeSingularValues



function [U,S,V] = fsvd(A, k, i, usePowerMethod)
% FSVD Fast Singular Value Decomposition 
% 
%   [U,S,V] = FSVD(A,k,i,usePowerMethod) computes the truncated singular
%   value decomposition of the input matrix A upto rank k using i levels of
%   Krylov method as given in [1], p. 3.
% 
%   If usePowerMethod is given as true, then only exponent i is used (i.e.
%   as power method). See [2] p.9, Randomized PCA algorithm for details.
% 
%   [1] Halko, N., Martinsson, P. G., Shkolnisky, Y., & Tygert, M. (2010).
%   An algorithm for the principal component analysis of large data sets.
%   Arxiv preprint arXiv:1007.5510, 0526. Retrieved April 1, 2011, from
%   http://arxiv.org/abs/1007.5510. 
%   
%   [2] Halko, N., Martinsson, P. G., & Tropp, J. A. (2009). Finding
%   structure with randomness: Probabilistic algorithms for constructing
%   approximate matrix decompositions. Arxiv preprint arXiv:0909.4061.
%   Retrieved April 1, 2011, from http://arxiv.org/abs/0909.4061.
% 
%   See also SVD.
% 
%   Copyright 2011 Ismail Ari, http://ismailari.com.

        if nargin < 3
                i = 1;
        end

        % Take (conjugate) transpose if necessary. It makes H smaller thus
        % leading the computations to be faster
        if size(A,1) < size(A,2)
                A = A';
                isTransposed = true;
        else
                isTransposed = false;
        end

        n = size(A,2);
        l = k + 2;

        % Form a real n×l matrix G whose entries are iid Gaussian r.v.s of zero
        % mean and unit variance
        G = randn(n,l);


        if nargin >= 4 && usePowerMethod
                % Use only the given exponent
                H = A*G;
                for j = 2:i+1
                        H = A * (A'*H);
                end
        else
                % Compute the m×l matrices H^{(0)}, ..., H^{(i)}
                % Note that this is done implicitly in each iteration below.
                H = cell(1,i+1);
                H{1} = A*G;
                for j = 2:i+1
                        H{j} = A * (A'*H{j-1});
                end

                % Form the m×((i+1)l) matrix H
                H = cell2mat(H);
        end

        % Using the pivoted QR-decomposiion, form a real m×((i+1)l) matrix Q
        % whose columns are orthonormal, s.t. there exists a real
        % ((i+1)l)×((i+1)l) matrix R for which H = QR.  
        % XXX: Buradaki column pivoting ile yapılmayan hali.
        [Q,~] = qr(H,0);

        % Compute the n×((i+1)l) product matrix T = A^T Q
        T = A'*Q;

        % Form an SVD of T
        [Vt, St, W] = svd(T,'econ');

        % Compute the m×((i+1)l) product matrix
        Ut = Q*W;

        % Retrieve the leftmost m×k block U of Ut, the leftmost n×k block V of
        % Vt, and the leftmost uppermost k×k block S of St. The product U S V^T
        % then approxiamtes A. 

        if isTransposed
                V = Ut(:,1:k);
                U = Vt(:,1:k);     
        else
                U = Ut(:,1:k);
                V = Vt(:,1:k);
        end
        S = St(1:k,1:k);
end



if __name__ == '__main__':
        print
        print "Higher order singular value decomposition routines"
        print
        print "as introduced in:"
        print "    Lieven de Lathauwer, Bart de Moor, Joos Vandewalle,"
        print "    'A multilinear singular value decomposition',"
        print "    SIAM J. Matrix Anal. Appl. 21 (4), 2000, 1253-1278"

        print
        print "Here are some worked examples from the paper."

        print
        print
        print "Example 1 from the paper (p. 1256)"

        A = np.zeros((3,2,3))

        A[0,0,0]=A[0,0,1]=A[1,0,0]=1
        A[1,0,1]=-1
        A[1,0,2]=A[2,0,0]=A[2,0,2]=A[0,1,0]=A[0,1,1]=A[1,1,0]=2
        A[1,1,1]=-2
        A[1,1,2]=A[2,1,0]=A[2,1,2]=4
        #other elements implied zero

        #test: compute unfold(A,1)
        print
        print "The input tensor is:"
        print A
        print
        print "Its unfolding along the first axis is:"
        print unfold(A,1)

        """
        print
        print
        print "Example 2 from the paper (p. 1257)""

        A = np.zeros((2,2,2))
        A[0,0,0] = A[1,1,0] = A[0,0,1] = 1
        #other elements implied zero
        """

        """
        print
        print
        print "Example 3 from the paper (p. 1257)""

        A = np.zeros((2,2,2))
        A[1,0,0] = A[0,1,0] = A[0,0,1] = 1
        #other elements implied zero
        """
        print
        print
        print "Example 4 from the paper (pp. 1264-5)"
        A = np.zeros((3,3,3))

        A[:,0,:] = np.asmatrix([[0.9073, 0.8924, 2.1488],
        [0.7158, -0.4898, 0.3054],
        [-0.3698, 2.4288, 2.3753]]).transpose()

        A[:,1,:] = np.asmatrix([[1.7842, 1.7753, 4.2495],
        [1.6970, -1.5077, 0.3207],
        [0.0151, 4.0337, 4.7146]]).transpose()

        A[:,2,:] = np.asmatrix([[2.1236, -0.6631, 1.8260],
        [-0.0740, 1.9103, 2.1335],
        [1.4429, -1.7495,-0.2716]]).transpose()

        print "The input tensor has matrix unfolding along axis 1:"
        print unfold(A,1)
        print

        U, S, D = HOSVD(A)

        print "The left n-mode singular matrices are:"
        print U[0]
        print
        print U[1]
        print
        print U[2]
        print

        print "The core tensor has matrix unfolding along axis 1:"
        print unfold(S, 1)
        print

        print "The n-mode singular values are:"
        print D[0]
        print D[1]
        print D[2]
'''


    
if __name__ == "__main__":
    
    pass



import os
from setuptools import setup, find_packages

VERSION = '0.1.1'


setup(
    name = 'stamps',
    packages = find_packages(exclude=['contrib', 'docs', 'tests']),
    license = 'MIT',
    version = VERSION,
    description = 'Spatial Temporal Analysis and Mapping Python Suite',
    install_requires = [
        'numpy>=1.13', 'scipy>=0.15', 'pandas>=0.22', 'six>=1.11'],
    author = 'stemlab',
    author_email = 'stemlab689@gmail.com',
    url = 'https://github.com/stemlab689/stamps',
    download_url = 'https://github.com/stemlab689/stamps/archive/{v}.tar.gz'\
        .format(v=VERSION),
    keywords = 'analysis mapping stamps',
    classifiers = [
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        ]
    )



# -*- coding: utf-8 -*-
'''
Created on 2011/11/12

@author: ksj
'''
import numpy


def findzfromgriddata(x, y, t, s_grid, t_grid, z_grid, DataObj = None):
    '''
    x: raw by 1 numpy array
    y: ditto
    t: ditto
    s_grid: raw by 2 numpy array
    t_grid: 1 by col numpy array
    z_grid: raw by col numpy array
    DataObj: for GUI use
    
    return z, raw by 1 numpy array,or return False if fail
    '''
    
    if not DataObj:
        from nousedataobj import NoUseDataObj
        DataObj = NoUseDataObj()
        
    title = DataObj.getProgressText()
    
    z = numpy.empty(x.shape)
    z[:] = numpy.nan
    
    DataObj.setProgressRange(0,len(s_grid))
    DataObj.setCurrentProgress(0, title + "\n- Find GridZ to RawZ...")
    
    for index_s, xy in enumerate(s_grid):
        if not DataObj.wasProgressCanceled():
            index_x = numpy.where(x == xy[0])[0]
            yyyy = y[index_x]
            index_y = numpy.where(yyyy == xy[1])[0]
            select_index = index_x[index_y]
            
            for index_tt, tt in enumerate(t_grid[0]):
                if numpy.isnan(z_grid[index_s, index_tt]): # no value
                    pass
                else:
                    tttt=t[select_index]
                    index_t = numpy.where(tttt == tt)[0]
                    last_index = index_t
                    
                    if len(last_index) == 0:
                        pass # not find
                    elif len(last_index) == 1:
                        z[select_index[last_index]] = z_grid[index_s, index_tt]
            DataObj.setCurrentProgress(index_s + 1)
        else:
            return False
    DataObj.setCurrentProgress(text = title)
    return z
    
if __name__ == "__main__":
    x = numpy.array([1,3,1,2,3.,6],ndmin=2).T
    y = numpy.array([2,2,1,2,4.,6],ndmin=2).T
    t = numpy.array([1,1,2,2,2.,1],ndmin=2).T
    s_grid = numpy.array([[1.,1],
                          [1,2],
                          [2,2],
                          [3,2],
                          [3,4]])
    
    t_grid = numpy.array([[1,2.]])
    z_grid = numpy.array([[numpy.nan,72.],
                          [2,numpy.nan],
                          [numpy.nan,2],
                          [55,numpy.nan],
                          [9999,998.]])
    
    z = findzfromgriddata(x, y, t, s_grid, t_grid, z_grid)
    print numpy.hstack((x,y,t,z))


# -*- coding: utf-8 -*-
'''
Created on 2012/7/6

@author: KSJ
'''
# import os
import numpy
import matplotlib.pyplot as plt

from .stcov import stcov
from ..stest import stmean

def polarstcovmap(cMS,tME,Zst,rLag,rLagTol,tLag=None,tLagTol=None,angLag=None,angTol=None,
                  plot=True):
  '''
  Plot polar map for empirical spatial covariance of space-time data
  SYNTAX :

  polarstcovmap(cMS,tME,Zst,rLag,rLagTol,tLag=None,tLagTol=None,angLag=None
                ,angTol=None,plot=True)

  INPUT : 

  cMS       ns by d       
  tME       1 by nt       
  Zst       ns by nt      Data
  rLag      nls by 1      1D array of vector with the r lags    
  rLagTol   nls by 1      1D array of vector with the tolerance for the r lags
  angLag    na by 1       1D array of angles to be evaluated (in radian). The 
                          two ends should be [-pi/2,pi/2) 
                          The default is [-pi/2:30/180*pi:pi/2)
  angTol    na by 1       1D array of angle tolerance around the each of angles
                          the default is the array of 15/180*pi
  plot      bool          True for plot the polar covariance. Default is True                       
                         
  OUTPUT:
  CC        list          a list of na directional empirical covariances. Each 
                          has shape of nls by nlt
  CCn       list          a list of na directional empirical covariances. Each 
                          has shape of nls by nlt
  lagr      nls by nlt    meshed spatial distances
  lagt      nls by nlt    meshed temporal lags                    
  angLag    na by 1       1D array of the diretions being evaluated      

  Remark: 
  For the purposes of plotting polar empirical covariance,               
  '''
  
  if angLag is None:
    ang_inic=-numpy.pi/2
    ang_step=30/180.*numpy.pi
    angLag=numpy.arange(ang_inic,numpy.pi/2,ang_step)
    angTol=numpy.ones(angLag.shape)*ang_step/2
  
  CC=[]
  CCn=[]
  CC4plot=numpy.empty([rLag.size,0])   
  for i,ang in enumerate(angLag):
    if tLag is not None:
      C,Cn,lagr,lagt=stcov(cMS,tME,Zst,rLag,rLagTol,tLag,tLagTol,ang=ang,angtol=angTol[i])
    else:
      C,Cn,lagr,lagt=stcov(cMS,tME,Zst,rLag,rLagTol,ang=ang,angtol=angTol[i])
    CC.append(C)
    CCn.append(Cn)
    CC4plot=numpy.hstack((CC4plot,C[:,0:1]))       
  
  if plot is True:
    idx=numpy.where(numpy.isnan(CC4plot))
    if idx[0].size>0:  
      CC3=numpy.hstack((CC4plot,numpy.hstack((CC4plot,CC4plot))))
      angLag3=numpy.hstack((angLag-numpy.pi,numpy.hstack((angLag,angLag+numpy.pi)))) 
      try:
        CC1=stmean.stmeaninterp(lagr,angLag3,CC3,lagr,angLag,method='linear')
      except:  
        CC1=stmean.stmeaninterp(lagr,angLag3,CC3,lagr,angLag,method='nearest')
      for k in xrange(numpy.int(idx[0].size)):
        CC4plot[idx[0][k],idx[1][k]]=CC1[idx[0][k],idx[1][k]]
      
    angLag4plot=numpy.hstack((angLag,angLag+numpy.pi))
    CC4plot=numpy.hstack((CC4plot,CC4plot))
    angLag4plot=numpy.append(angLag4plot,angLag4plot[-1]+angTol[0]*2)
    CC4plot=numpy.hstack((CC4plot,CC4plot[:,0:1]))
    
    angLag4plot=angLag4plot-angTol[0]
    theta,radius = numpy.meshgrid(angLag4plot, rLag) #rectangular plot of polar data
    fig=plt.figure()
    ax = fig.add_subplot(111,polar=True)
    im=ax.pcolor(theta, radius, CC4plot,cmap='hot_r')  
    plt.colorbar(im)  
    plt.show()  
  
  return CC,CCn,lagr,lagt,angLag

          
def iso2aniso(ciso,angle,ratio):
  '''
% iso2aniso                 - convert isotropic to anisotropic coordinates 
%
% Do the transformation which is reciprocal to the transformation made by
% the aniso2iso.m function, so that applying successively both transformation
% has no effect on the coordinates. The function maps a set of coordinates
% in an isotropic space into a set of coordinates in an anisotropic one. 
%
% SYNTAX :
%
% [c]=iso2aniso(ciso,angle,ratio); 
%
% INPUT :
%
% ciso    n by d   An 2D array of coordinates for the locations in the isotropic
%                  space. A line corresponds to the vector of coordinates at a
%                  location, so that the number of columns corresponds to the
%                  dimension of the space. Only two dimensional or three dimensional
%                  space coordinates can be processed by this function.
% angle   1 by d-1 vector of angle values that characterize the anisotropy. 
%                  In a two dimensional space, angle is the trigonometric angle
%                  between the horizontal axis and the principal axis of the
%                  ellipse. In a three dimensional space, spherical coordinates
%                  are used, such that angle(1) is the horizontal trigonometric
%                  angle and angle(2) is the vertical trigonometric angle for the
%                  principal axis of the ellipsoid. All the angles are measured
%                  counterclockwise in degrees and are between -90 and 90.
% ratio   1 by d-1 vector that characterize the ratio for the length of the axes
%                  for the ellipse (in 2D) or ellipsoid (in 3D). In a two dimensional
%                  space, ratio is the length of the secondary axis of the ellipse
%                  divided by the length of the principal axis, so that ratio in [0,1]. In a
%                  three dimensional space, ratio(1) is the length of the second
%                  axis of the ellipsoid divided by the length of the principal axis, 
%                  whereas ratio(2) is length of the third axis of the ellipsoid
%                  divided by the length of the principal axis, so that ratio(1) and
%                  ratio(2) are both in the range of [0,1].
%
% OUTPUT :
%
% c       n by d   An 2D array of coordinates having the same size as ciso, that gives the new coordinates
%                  in the anisotropic space.
%
% NOTE :
%
% It is possible to specify an additional index vector, taking integer values from 1
% to nv. The values in index specifies which of the nv variable is known at each one
% of the corresponding coordinates. The ciso matrix of coordinates and the index vector
% are then grouped together using the MATLAB cell array notation, so that ciso={ciso,index}.
% This allows to perform the same coordinate transformation at once on a set of possibly
% different variables. The output variable c is then also a cell array that contains
% both the new matrix of coordinates and the index vector.
  '''
    
  # Determine the dimension of the space and set ratio 
  if type(ciso) is numpy.ndarray:  
    d=ciso.shape[1]
  else:
    ciso=numpy.array(ciso,ndmin=2)
    d=ciso.shape[1]
  angle=angle*2*numpy.pi/360
  angle=-angle

  # When d<2 or d>3, error

  if (d<2) or (d>3):
    print 'iso2aniso function requires coordinates in a 2D or 3D space'
    return

  # Case for d=2

  if d==2:
    R=numpy.array([[numpy.cos(angle),numpy.sin(angle)],
                   [-numpy.sin(angle),numpy.cos(angle)]])
    ciso[:,1]=ciso[:,1]*ratio
    c=ciso.dot(R.T)
    

  # Case for d=3

  if d==3:
    phi=angle[0]
    teta=angle[1]
    ratioy=ratio[0]
    ratioz=ratio[1]

    R1=numpy.array([[numpy.cos(phi),numpy.sin(phi),0],
                    [-numpy.sin(phi),numpy.cos(phi),0],
                    [0,0,1]])
    R2=numpy.array([[numpy.cos(teta),0,numpy.sin(teta)],
                    [0,1,0],
                    [-numpy.sin(teta),0,numpy.cos(teta)]])
    R=(R2.T).dot(R1.T)
    ciso[:,1]=ciso[:,1]*ratioy
    ciso[:,2]=ciso[:,2]*ratioz    
    c=ciso.dot(R)

    
  return c


def aniso2iso(c,angle,ratio):
  '''
% aniso2iso                 - convert anisotropic to isotropic coordinates (Jan 1,2001)
%
% Transform a set of two dimensional or three dimensional coordinates
% using rotations and dilatations of the axes, in order to map an
% anisotropic space into an isotropic one. The geometric anisotropy
% is characterized by the angle(s) of the principal axis of the ellipse
% (in 2D) or ellipsoid (in 3D), and by the ratio(s) of the principal axis
% length by the other axes lengths. Using this function, an ellipse
% (ellipsoid) is thus mapped into a circle (sphere) having as radius the
% length of the principal axis. The transformation consist in a rotation
% of the axes followed by a dilatation. 
% 
% SYNTAX :
%
% [ciso]=aniso2iso(c,angle,ratio); 
%
% INPUT :
%
% c       n by d   matrix of coordinates for the locations in the anisotropic
%                  space. A line corresponds to the vector of coordinates at a
%                  location, so that the number of columns corresponds to the
%                  dimension of the space. Only two dimensional or three dimensional
%                  space coordinates can be processed by this function.
% angle   1 by d-1 vector of angle values that characterize the anisotropy. 
%                  In a two dimensional space, angle is the trigonometric angle
%                  between the horizontal axis and the principal axis of the
%                  ellipse. In a three dimensional space, spherical coordinates
%                  are used, such that angle(1) is the horizontal trigonometric
%                  angle and angle(2) is the vertical trigonometric angle for the
%                  principal axis of the ellipsoid. All the angles are measured
%                  counterclockwise in degrees and are between -90∞ and 90∞.
% ratio   1 by d-1 vector that characterize the ratio for the length of the axes
%                  for the ellipse (in 2D) or ellipsoid (in 3D). In a two dimensional
%                  space, ratio is the length of the secondary axis of the ellipse
%                  divided by the length of the principal axis, so that ratio in [0,1]. In a
%                  three dimensional space, ratio(1) is the length of the second
%                  axis of the ellipsoid divided by the length of the principal axis, 
%                  whereas ratio(2) is length of the third axis of the ellipsoid
%                  divided by the length of the principal axis, so that ratio(1) and
%                  ratio(2) are both in the range of [0,1].
%
% OUTPUT :
%
% ciso    n by d   matrix having the same size as c, that gives the new coordinates
%                  in the isotropic space.
%
% NOTE :
%
% It is possible to specify an additional index vector, taking integer values from 1
% to nv. The values in index specify which one of the nv variable is known at each one
% of the corresponding coordinates. The c matrix of coordinates and the index vector
% are then grouped together using the MATLAB cell array notation, so that c={c,index}.
% This allows to perform the same coordinate transformation at once on a set of possibly
% different variables. The output variable ciso is then also a cell array that contains
% both the new matrix of coordinates and the index vector.
  '''
  
  # Determine the dimension of the space and set angle
  if type(c) is numpy.ndarray:  
    d=c.shape[1]
  else:
    c=numpy.array(c,ndmin=2)
    d=c.shape[1]
  angle=angle*2*numpy.pi/360;

  # When d<2 or d>3, error

  if (d<2) or (d>3):
    print 'aniso2iso requires coordinates in a 2D or 3D space'
    return

  # Case for d=2

  if d==2:
    R=numpy.array([[numpy.cos(angle),numpy.sin(angle)],
                  [-numpy.sin(angle),numpy.cos(angle)]])
    ciso=c.dot(R.T)
    ciso[:,1]=ciso[:,1]*1./ratio

  # Case for d=3

  if d==3:
    phi=angle[0]
    teta=angle[1]
    ratioy=ratio[0]
    ratioz=ratio[1]
  
    R1=numpy.array([[numpy.cos(phi),numpy.sin(phi),0],
                    [-numpy.sin(phi),numpy.cos(phi),0],
                    [0,0,1]]) 
    R2=numpy.array([[numpy.cos(teta),0,numpy.sin(teta)],
                    [0,1,0],
                    [-numpy.sin(teta),0,numpy.cos(teta)]])
    R=(R1.T).dot(R2.T)
    ciso=c.dot(R)
    ciso[:,1]=ciso[:,1]*1./ratioy
    ciso[:,2]=ciso[:,2]*1./ratioz
  
  return ciso   
